/**
 *  This file is part of
 *  
 *  Solenopsis 2: A Framework for the Development of Ant Algorithms
 *  
 *  Copyright (C) 2006-2009 Amos Brocco <amos.brocco@unifr.ch>
 *  						Pervasive and Artificial Intelligence Research Group
 *  						Department of Informatics DIUF
 *  						University of Fribourg, Fribourg
 *  						Switzerland
 *
 *  Solenopsis 2 is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  Solenopsis 2 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with Solenopsis 2.  If not, see <http://www.gnu.org/licenses/>.
 *  
 */
package bebop.client;

import interfaces.vm.util.ISerializer;

import java.io.BufferedOutputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.net.InetAddress;
import java.net.Socket;
import java.net.UnknownHostException;
import java.util.ArrayList;
import java.util.List;

import logging.Logger;
import serialization.Serializer;
import bebop.lib.IMessage;

public class Client {
	
	private static List<SendAction> actions = new ArrayList<SendAction>();
	
	static class SendAction {
		public IMessage msg;
		
		public SendAction(IMessage msg) {
			this.msg = msg;
		}
	}

	static class PostmanThread implements Runnable {

		public void run() {
			for (;;) {
				try {
					synchronized(actions) {
						for (SendAction sa : actions) {
							sendMessageNow(sa.msg);
						}
						actions.clear();
					}
					Thread.sleep(10);
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		}
	}
	
	public static void init() {
		new Thread(new PostmanThread()).start();
	}
	
	public static void sendMessage(IMessage msg) {
		SendAction s = new SendAction(msg);
		synchronized(actions) {
			actions.add(s);
		}
	}

	public static synchronized void sendMessageNow(IMessage m)
			throws UnknownHostException, IOException {
		Socket s = null;
		try {
			ByteArrayOutputStream out = new ByteArrayOutputStream();
			List<Object> l = new ArrayList<Object>();
			l.add(m.getRecipient().toString());
			l.add(m.getContents());
			ISerializer ser = new Serializer(out);
			try {
				ser.print(l);
			} catch (Exception e) {
				e.printStackTrace();
			}
			String ba = out.toString();
			s = new Socket(InetAddress.getByName(m.getRecipient().getHost()), m.getRecipient().getPort());
			s.setKeepAlive(false);
			s.setSoTimeout(30000);
			OutputStream oStream = new BufferedOutputStream(s.getOutputStream());
			oStream.write(ba.getBytes());
			oStream.write('\n');
			oStream.flush();
			oStream.close();
		} catch (IOException e) {
			Logger.warning("Failed to send message: %s\n", m.toString());
			e.printStackTrace();
			throw e;
		} finally {
			if (s != null)
				s.close();
		}
		
	}

}
