/**
 *  This file is part of
 *  
 *  Solenopsis 2: A Framework for the Development of Ant Algorithms
 *  
 *  Copyright (C) 2006-2009 Amos Brocco <amos.brocco@unifr.ch>
 *  						Pervasive and Artificial Intelligence Research Group
 *  						Department of Informatics DIUF
 *  						University of Fribourg, Fribourg
 *  						Switzerland
 *
 *  Solenopsis 2 is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  Solenopsis 2 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with Solenopsis 2.  If not, see <http://www.gnu.org/licenses/>.
 *  
 */
package bebop.server;

import java.io.IOException;
import java.net.InetAddress;
import java.net.ServerSocket;
import java.net.Socket;
import java.net.UnknownHostException;
import java.util.Observer;

import logging.Logger;


import bebop.buffer.Buffer;
import bebop.buffer.IBuffer;
import bebop.client.Client;

public class Server implements Runnable {

	private ServerSocket s;
	private int serverport = 7893;
	private IBuffer buffer;
	private String host;

	public Server(IBuffer buffer, String host, int port) throws IOException {
		this.buffer = buffer;
		this.serverport = port;
		this.host = host;
		s = new ServerSocket(serverport);
		Logger.debug("Bebox Mailbox Server serving host %s on port %d", host, serverport);
		Client.init();
	}

	public void run() {
		while (true) {
			fetchAndExecuteMessage();
		}
	}

	public synchronized void fetchAndExecuteMessage() {
		Socket incoming = null;
		try {
			incoming = s.accept();
			new Thread(new ServingThread(incoming, buffer)).start();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	public static Server start(IBuffer buffer, String host, int port) throws IOException {
		Server server = new Server(buffer, host, port);
		new Thread(server).start();
		return server;
	}

	public static IBuffer start(Observer observer, String host, int port) throws IOException {
		Buffer buffer = new Buffer();
		buffer.addObserver(observer);
		Server server = new Server(buffer, host, port);
		new Thread(server).start();
		return buffer;
	}
	
	public String getHost() {
		if (host == null) {
			try {
				return InetAddress.getLocalHost().getHostAddress();
			} catch (UnknownHostException e) {
				return "localhost";
			}
		} else {
			return host;
		}
	}
	
	public int getPort() {
		 return s.getLocalPort();
	}

}
