/**
 *  This file is part of
 *  
 *  Solenopsis 2: A Framework for the Development of Ant Algorithms
 *  
 *  Copyright (C) 2006-2009 Amos Brocco <amos.brocco@unifr.ch>
 *  						Pervasive and Artificial Intelligence Research Group
 *  						Department of Informatics DIUF
 *  						University of Fribourg, Fribourg
 *  						Switzerland
 *
 *  Solenopsis 2 is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  Solenopsis 2 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with Solenopsis 2.  If not, see <http://www.gnu.org/licenses/>.
 *  
 */
package bebop.server;

import java.io.BufferedInputStream;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.net.Socket;
import java.util.List;
import java.util.Map;

import serialization.DeSerializer;
import bebop.buffer.IBuffer;
import bebop.lib.Message;

public class ServingThread implements Runnable {

	private int MAX_MESSAGE_SIZE = Integer.MAX_VALUE; // Limit the size

	private Socket incoming = null;

	private IBuffer buffer;
	
	private ByteArrayOutputStream baos = new ByteArrayOutputStream(Short.MAX_VALUE);
	
	private byte[] temp = new byte[Short.MAX_VALUE];

	public ServingThread(Socket incoming, IBuffer buffer) {
		this.incoming = incoming;
		this.buffer = buffer;
	}

	@SuppressWarnings("unchecked")
	public Message processIncomingRequest() throws IOException {
		Message msg = null;
		
		baos.reset();
		
		int count = 0;
		int read = 0;

		incoming.setSoTimeout(0);

		BufferedInputStream bis = new BufferedInputStream(incoming.getInputStream());
		
		while ((read = bis.read(temp)) > 0) {
			baos.write(temp, 0, read);
			count += read;
			if (count > MAX_MESSAGE_SIZE) {
				logging.Logger.warning("Incoming message too large, chopped!");
				bis.close();
				return null;
			}
		}
		     
		bis.close();
		
		ByteArrayInputStream bais = new ByteArrayInputStream(baos.toByteArray());
		
		DeSerializer ds = null;
		ds = new DeSerializer(bais);
		bais.close();
		try {
			List data = (List) ds.get();
			msg = new Message((String) data.get(0), (Map) data.get(1));
		} catch (Exception e) {
			return null;
		}

		
		return msg;
	}

	public void run() {
		Message msg = null;
		try {
			msg = processIncomingRequest();
		} catch (IOException e) {
			e.printStackTrace();
		} finally {
			try {
				incoming.close();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		if (msg != null) {
				this.buffer.pushMessage(msg);
		}
	}

}
