/**
 *  This file is part of
 *  
 *  Solenopsis 2: A Framework for the Development of Ant Algorithms
 *  
 *  Copyright (C) 2006-2009 Amos Brocco <amos.brocco@unifr.ch>
 *  						Pervasive and Artificial Intelligence Research Group
 *  						Department of Informatics DIUF
 *  						University of Fribourg, Fribourg
 *  						Switzerland
 *
 *  Solenopsis 2 is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  Solenopsis 2 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with Solenopsis 2.  If not, see <http://www.gnu.org/licenses/>.
 *  
 */
package cli;


public class BooleanOption implements IBooleanOption {
	
	private String description;
	private String shortLabel;
	private String longLabel;
	private boolean value;
	private boolean defaultValue;
	private boolean isMandatory;
	
	public BooleanOption(String shortL, String longL, String desc, boolean defValue, boolean mandatory) {
		shortLabel = shortL;
		longLabel = longL;
		defaultValue = defValue;
		value = defValue;
		description = desc;
		isMandatory = mandatory;
	}

	public String getDefault() {
		return new Boolean(defaultValue).toString();
	}

	public String getDescription() {
		return description;
	}

	public String getLongLabel() {
		return longLabel;
	}

	public String getShortLabel() {
		return shortLabel;
	}

	public int getSkip() {
		return 2;
	}

	public void setValue(String string) {
		value = string.equals("false") ? false : true;
	}

	public boolean isMandatory() {
		return isMandatory;
	}
	
	public String getType() {
		return "[true|false]";
	}
	
	public boolean isValidValue(String value) {
		return value.equals("true") || value.equals("false");
	}

	public boolean ignoreInvalid() {
		return true;
	}

	public Boolean getValue() {
		return value;
	}

}
