/**
 *  This file is part of
 *  
 *  Solenopsis 2: A Framework for the Development of Ant Algorithms
 *  
 *  Copyright (C) 2006-2009 Amos Brocco <amos.brocco@unifr.ch>
 *  						Pervasive and Artificial Intelligence Research Group
 *  						Department of Informatics DIUF
 *  						University of Fribourg, Fribourg
 *  						Switzerland
 *
 *  Solenopsis 2 is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  Solenopsis 2 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with Solenopsis 2.  If not, see <http://www.gnu.org/licenses/>.
 *  
 */
package cli;


public class FloatOption implements IFloatOption {
	
	private String description;
	private String shortLabel;
	private String longLabel;
	private float value;
	private float defaultValue;
	private boolean isMandatory;
	
	public FloatOption(String shortL, String longL, String desc, float defValue, boolean mandatory) {
		shortLabel = shortL;
		longLabel = longL;
		defaultValue = defValue;
		description = desc;
		value = defaultValue;
		isMandatory = mandatory;
	}

	public String getDefault() {
		return new Float(defaultValue).toString();
	}

	public String getDescription() {
		return description;
	}

	public String getLongLabel() {
		return longLabel;
	}

	public String getShortLabel() {
		return shortLabel;
	}

	public int getSkip() {
		return 2;
	}

	public void setValue(String string) {
		value = Float.parseFloat(string);
	}

	public Float getValue() {
		return value;
	}

	public boolean isMandatory() {
		return isMandatory;
	}

	public String getType() {
		return "float";
	}

	public boolean isValidValue(String value) {
		try {
			this.value = Float.parseFloat(value);
			return true;
		} catch(NumberFormatException e) {
			return false;
		}
	}
	
	public boolean ignoreInvalid() {
		return false;
	}

}
