/**
 *  This file is part of
 *  
 *  Solenopsis 2: A Framework for the Development of Ant Algorithms
 *  
 *  Copyright (C) 2006-2009 Amos Brocco <amos.brocco@unifr.ch>
 *  						Pervasive and Artificial Intelligence Research Group
 *  						Department of Informatics DIUF
 *  						University of Fribourg, Fribourg
 *  						Switzerland
 *
 *  Solenopsis 2 is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  Solenopsis 2 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with Solenopsis 2.  If not, see <http://www.gnu.org/licenses/>.
 *  
 */
package core.vm.lib;

public class VMMath {
	
	//int -> long -> float -> double
	
	@SuppressWarnings("unchecked")
	private static Class[] types = {Integer.class, Long.class, Float.class, Double.class};
	
	private static int type(Number x) {
		if (x instanceof Double)
			return 3;
		else if (x instanceof Float)
			return 2;
		else if (x instanceof Long)
			return 1;
		else if (x instanceof Integer)
			return 0;
		else
			return 0;
	}
	
	private static int uType(Number x, Number y) {
		int typex = type(x);
		int typey = type(y);
		return Math.max(typex, typey);
	}

	@SuppressWarnings("unchecked")
	public static Number sum(Number x, Number y) {
		Class t = types[uType(x,y)];
		if (Integer.class.equals(t)) {
			return x.intValue() + y.intValue();
		} else if (Float.class.equals(t)) {
			return x.floatValue() + y.floatValue();
		} else if (Long.class.equals(t)) {
			return x.longValue() + y.longValue();
		} else {
			return x.doubleValue() + y.doubleValue();
		}
	}
	
	@SuppressWarnings("unchecked")
	public static Number mul(Number x, Number y) {
		Class t = types[uType(x,y)];
		if (Integer.class.equals(t)) {
			return x.intValue() * y.intValue();
		} else if (Float.class.equals(t)) {
			return x.floatValue() * y.floatValue();
		} else if (Long.class.equals(t)) {
			return x.longValue() * y.longValue();
		} else {
			return x.doubleValue() * y.doubleValue();
		}
	}
	
	@SuppressWarnings("unchecked")
	public static Number div(Number x, Number y) {
		Class t = types[uType(x,y)];
		if (Integer.class.equals(t)) {
			return x.intValue() / y.intValue();
		} else if (Float.class.equals(t)) {
			return x.floatValue() / y.floatValue();
		} else if (Long.class.equals(t)) {
			return x.longValue() / y.longValue();
		} else {
			return x.doubleValue() / y.doubleValue();
		}
	}
	
	@SuppressWarnings("unchecked")
	public static Number sub(Number x, Number y) {
		Class t = types[uType(x,y)];
		if (Integer.class.equals(t)) {
			return x.intValue() - y.intValue();
		} else if (Float.class.equals(t)) {
			return x.floatValue() - y.floatValue();
		} else if (Long.class.equals(t)) {
			return x.longValue() - y.longValue();
		} else {
			return x.doubleValue() - y.doubleValue();
		}
	}

}
