/**
 *  This file is part of
 *  
 *  Solenopsis 2: A Framework for the Development of Ant Algorithms
 *  
 *  Copyright (C) 2006-2009 Amos Brocco <amos.brocco@unifr.ch>
 *  						Pervasive and Artificial Intelligence Research Group
 *  						Department of Informatics DIUF
 *  						University of Fribourg, Fribourg
 *  						Switzerland
 *
 *  Solenopsis 2 is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  Solenopsis 2 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with Solenopsis 2.  If not, see <http://www.gnu.org/licenses/>.
 *  
 */
package core.vm.lib;

import interfaces.vm.valuetypes.ILambdaValue;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;

import com.fuzzbug.solenopsis.sdk.vm.IVMStackFrame;


import core.vm.valuetypes.LambdaValue;

public class VMStackFrame implements IVMStackFrame {

	private static final long serialVersionUID = -3055894012914425338L;

	private List<Object> localStack;

	private int returnPC;

	private Map<String,Object> localVariables;

	private ILambdaValue lambdaValue;
	
	private boolean local = false;
	
	public VMStackFrame() {
	}

	@SuppressWarnings("unchecked")
	public VMStackFrame(List<Object> source) {
		returnPC = ((Integer) source.get(0)).intValue();
		localVariables = (Map<String, Object>) source.get(1);
		localStack = (List<Object>) source.get(2);
		local = (Boolean) source.get(3);
		lambdaValue = source.get(4) instanceof LambdaValue ? (LambdaValue) source.get(4) : null;
	}

	public VMStackFrame(int pc, boolean local) {
		this(pc, null, local);
	}

	public VMStackFrame(int pc, ILambdaValue lambda, boolean local) {
		localStack = new LinkedList<Object>();
		localVariables = new HashMap<String,Object>();
		returnPC = pc;
		lambdaValue = lambda;
		this.local = local;
	}

	public int getReturnPC() {
		return returnPC;
	}

	public Object popValue() {
		return localStack.remove(localStack.size()-1);
	}

	public void pushValue(Object argument) {
		localStack.add(argument);
	}

	public void setValue(String key, Object value) {
		localVariables.put(key, value);
	}

	public Object getValue(String var) {
		return localVariables.get(var);
	}

	public Object getTOS() {
		if (localStack.isEmpty()) {
			return null;
		}
		return localStack.get(localStack.size()-1);
	}

	public Object toBaseValue() {
		List<Object> data = new ArrayList<Object>(5);
		data.add(new Integer(returnPC));
		data.add(localVariables);
		data.add(localStack);
		data.add(local);
		data.add(lambdaValue == null ? false : lambdaValue);
		return data;
	}

	public ILambdaValue getLambda() {
		return lambdaValue;
	}

	public boolean isEmpty() {
		return localStack.isEmpty();
	}

	public boolean isLocal() {
		return local;
	}

	@Override
	public int getSize() {
		return localStack.size();
	}

	@Override
	public Set<String> getVariables() {
		return localVariables.keySet();
	}
	
	public String toString() {
		StringBuilder str = new StringBuilder();
		str.append(super.toString());
		str.append(localVariables);
		for (Object f : localStack) {
			str.append(f.toString());
			str.append("\n");
		}
		return str.toString();
	}

}
