/**
 *  This file is part of
 *  
 *  Solenopsis 2: A Framework for the Development of Ant Algorithms
 *  
 *  Copyright (C) 2006-2009 Amos Brocco <amos.brocco@unifr.ch>
 *  						Pervasive and Artificial Intelligence Research Group
 *  						Department of Informatics DIUF
 *  						University of Fribourg, Fribourg
 *  						Switzerland
 *
 *  Solenopsis 2 is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  Solenopsis 2 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with Solenopsis 2.  If not, see <http://www.gnu.org/licenses/>.
 *  
 */
package core.vm.lib;

import interfaces.exe.ICodeProvider;

import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;

import com.fuzzbug.solenopsis.sdk.lib.IInstruction;
import com.fuzzbug.solenopsis.sdk.vm.IVMCode;
import com.fuzzbug.solenopsis.sdk.vm.IVMStack;
import com.fuzzbug.solenopsis.sdk.vm.IVMState;

import serialization.DeSerializer;

public class VMState implements IVMState {

	private static final long serialVersionUID = 5492694502816092760L;

	public static boolean serializeFullState = false;

	IVMStack stack = null;
	IVMCode code = null;
	Integer pc = 0;

	public VMState(IVMStack stack, IVMCode code, int pc) {
		this.stack = stack;
		this.code = code;
		this.pc = pc;
	}

	@SuppressWarnings("unchecked")
	public VMState(ICodeProvider c, InputStream serialized) throws Exception {
		DeSerializer ds = new DeSerializer(serialized);
		List<Object> stateTree = (List<Object>) ds.get();
		if (stateTree.size() == 4) {
			code = new VMCode((String) stateTree.get(0), (List<IInstruction>) stateTree.get(1));
			stack = new VMStack((List)stateTree.get(2));
			pc = (Integer) stateTree.get(3);
		} else {
			code = new VMCode(c, (String) stateTree.get(0));
			stack = new VMStack((List)stateTree.get(1));
			pc = (Integer) stateTree.get(2);
		}
	}

	public IVMStack getStack() {
		return stack;
	}

	public IVMCode getCode() {
		return code;
	}

	public int getPC() {
		return pc;
	}

	public Object toBaseValue() {
		List<Object> stateTree = new ArrayList<Object>(3);
		stateTree.add(code.getSourcePath());
		if (VMState.serializeFullState ) {
			stateTree.add(code.getInstructions());
		}
		stateTree.add(stack.toBaseValue());
		stateTree.add(new Integer(pc));
		return stateTree;
	}

	public String getSignature() {
		return this.toString();
	}

}
