/**
 *  This file is part of
 *  
 *  Solenopsis 2: A Framework for the Development of Ant Algorithms
 *  
 *  Copyright (C) 2006-2009 Amos Brocco <amos.brocco@unifr.ch>
 *  						Pervasive and Artificial Intelligence Research Group
 *  						Department of Informatics DIUF
 *  						University of Fribourg, Fribourg
 *  						Switzerland
 *
 *  Solenopsis 2 is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  Solenopsis 2 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with Solenopsis 2.  If not, see <http://www.gnu.org/licenses/>.
 *  
 */
package core.vm.valuetypes;

import interfaces.vm.valuetypes.ILambdaValue;

import java.util.Arrays;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import util.Dolly;

import com.fuzzbug.solenopsis.sdk.vm.IVMCode;
import com.fuzzbug.solenopsis.sdk.vm.IVMStack;


public class LambdaValue implements ILambdaValue {
	
	private static final long serialVersionUID = -2973434156205042353L;
	private Map<String,Object> bindings;
	private Integer lambdaStart;
	private IVMCode code;
		
	public LambdaValue(IVMCode code, Integer absCodeStart, Map<String, Object> bindings) {
		this.bindings = bindings;
		this.code = code;
		this.lambdaStart = absCodeStart;
	}

	public String getSignature() {
		return "lambda("+lambdaStart+"@"+code+")"+bindings;
	}

	public Object toBaseValue() {
		return this;
	}

	public boolean equals(Object nil) {
		if (nil instanceof LambdaValue)
			return ((LambdaValue) nil).getSignature().equals(getSignature());
		else
			return false;
	}

	public int hashCode() {
		return getSignature().hashCode();
	}

	public String toString() {
		return getSignature();
	}

	public Integer getLambdaStart() {
		return lambdaStart;
	}

	public Object getVariableValue(String identifier) {
		return bindings.get(identifier);
	}

	public List<Object> getVariables() {
		return Arrays.asList(bindings.keySet().toArray());
	}

	public Map<String, Object> getBindings() {
		return bindings;
	}

	public void updateBindings(IVMStack env) {
		Map<String, Object> newBindings = new HashMap<String, Object>();
		Iterator<String> keyIter = bindings.keySet().iterator();
		while(keyIter.hasNext()) {
			String key = keyIter.next();
			Object currentValue = env.getValue(key);
			newBindings.put(key, currentValue);
		}
		bindings = newBindings;
	}
	
	@SuppressWarnings("unchecked")
	public Object clone() {
		try {
			Map<String, Object> newBindings = (Map<String, Object>) Dolly.deepClone(bindings);
			return new LambdaValue(code, lambdaStart, newBindings);
		} catch (Exception e) {
			return this;
		}
	}

	@Override
	public IVMCode getCode() {
		return code;
	}

}
