/**
 *  This file is part of
 *  
 *  Solenopsis 2: A Framework for the Development of Ant Algorithms
 *  
 *  Copyright (C) 2006-2009 Amos Brocco <amos.brocco@unifr.ch>
 *  						Pervasive and Artificial Intelligence Research Group
 *  						Department of Informatics DIUF
 *  						University of Fribourg, Fribourg
 *  						Switzerland
 *
 *  Solenopsis 2 is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  Solenopsis 2 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with Solenopsis 2.  If not, see <http://www.gnu.org/licenses/>.
 *  
 */
package dot.codegenerator;

import java.util.List;

import dot.codegenerator.macros.Macro;
import dot.codegenerator.modules.IGenerator;
import dot.parser.nodes.INode;

public interface ICodeGenerator {

	public boolean generateCode(INode node) throws Exception;

	public void registerPlugin(IGenerator plugin);

	public void pushGlobalFunction(String fname);

	public boolean isLocalFunction(String fname);
	
	public boolean isGlobalFunction(String fname);

	public void pushLocalVariable(String vname);

	public void pushGlobalVariable(String vname);

	public boolean isVisibleVariable(String vname);
	
	public boolean isVisibleFunction(String vname);
	
	public boolean isLocalVariable(String vname);
	
	public void pushLambdaFrame();
	
	public void pushLambdaVar(String name);
	
	public void popLambdaFrame();
	
	public List<String> getLambdaVars();
	
	public void pushLoopBreakTarget(String label);
	
	public String getLoopBreakTarget();
	
	public String popLoopBreakTarget();
	
	public void pushLoopContinueTarget(String label);

	public String getLoopContinueTarget();

	public String popLoopContinueTarget();
	
	public boolean isGlobalVariable(String vname);
	
	public void popEnvironment();
	
	public void pushEnvironment();
	
	public String generateJumpTarget();
	
	public String generateTempVarName();
	
	public void pushLabel(String label);
	
	public boolean isValidLabel(String label);

	public Object getEnvironment();
	
	public String getCurrentFunctionBlock();
	
	public void popFunctionBlock();
	
	public void pushFunctionBlock(String fname);
	
	public void pushMacro(Macro macro);

	public INode expand(INode node) throws Exception;
	
	public void redefine(String genName, String newName);
	
	public String getFunctionName(String name);
	
	public void undefine(String genName);
	
	public void overload(String genName, String newName);

	public void setContext(String string);
	
	public String getContext();
}
