/**
 *  This file is part of
 *  
 *  Solenopsis 2: A Framework for the Development of Ant Algorithms
 *  
 *  Copyright (C) 2006-2009 Amos Brocco <amos.brocco@unifr.ch>
 *  						Pervasive and Artificial Intelligence Research Group
 *  						Department of Informatics DIUF
 *  						University of Fribourg, Fribourg
 *  						Switzerland
 *
 *  Solenopsis 2 is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  Solenopsis 2 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with Solenopsis 2.  If not, see <http://www.gnu.org/licenses/>.
 *  
 */
package dot.codegenerator;


import java.util.ArrayList;
import java.util.List;
import java.util.Stack;

import com.fuzzbug.solenopsis.sdk.lib.IBaseSerializable;
import com.fuzzbug.solenopsis.sdk.lib.IConstants;
import com.fuzzbug.solenopsis.sdk.lib.IInstruction;
import com.fuzzbug.solenopsis.sdk.lib.IConstants.OPCODES;

import dot.parser.nodes.INode;

@SuppressWarnings("unchecked")
public class Instruction implements IBaseSerializable, IInstruction {

	private static final long serialVersionUID = 1L;

	IConstants.OPCODES opcode;

	ArrayList<Object> parameters;

	Stack<ArrayList> lists;

	ArrayList<Object> currentList;
	
	private INode node;

	public Instruction(IConstants.OPCODES op, INode node) {
		opcode = op;
		lists = new Stack();
		parameters = new ArrayList();
		currentList = parameters;
		this.node = node;
	}
	
	public Instruction(List data) {
		opcode = (OPCODES) data.get(0);
		parameters = (ArrayList<Object>) data.get(1);
	}
	
	public INode getNode() {
		return node;
	}

	public void pushParameter(Number number) {
		currentList.add(number);
	}

	public void openList() {
		lists.push(new ArrayList());
		currentList = (ArrayList) lists.lastElement();
	}
	
	public void closeList() {
		parameters.add(lists.pop());
		if (lists.empty())
			currentList = parameters;
		else
			currentList = (ArrayList) lists.lastElement();
	}

	public void pushParameter(String str) {
		currentList.add(str);
	}

	public void pushNil() {
		currentList.add(new Boolean(false));
	}

	/* (non-Javadoc)
	 * @see dot.codegenerator.IInstruction#getParameters()
	 */
	public ArrayList getParameters() {
		return parameters;
	}

	/* (non-Javadoc)
	 * @see dot.codegenerator.IInstruction#getOpCode()
	 */
	public OPCODES getOpCode() {
		return opcode;
	}

	public void setOpCode(IConstants.OPCODES op) {
		opcode = op;
	}
	
	public String toString() {
		return String.format(" %s ! %s", opcode.toString(), parameters.toString());
	}

	@Override
	public Object toBaseValue() {
		ArrayList tmp = new ArrayList<Object>();
		tmp.add(opcode);
		tmp.add(parameters);
		return tmp;
	}

}
