/**
 *  This file is part of
 *  
 *  Solenopsis 2: A Framework for the Development of Ant Algorithms
 *  
 *  Copyright (C) 2006-2009 Amos Brocco <amos.brocco@unifr.ch>
 *  						Pervasive and Artificial Intelligence Research Group
 *  						Department of Informatics DIUF
 *  						University of Fribourg, Fribourg
 *  						Switzerland
 *
 *  Solenopsis 2 is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  Solenopsis 2 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with Solenopsis 2.  If not, see <http://www.gnu.org/licenses/>.
 *  
 */
package dot.codegenerator.modules;


import java.util.List;

import com.fuzzbug.solenopsis.sdk.lib.IConstants;
import com.fuzzbug.solenopsis.sdk.lib.IInstruction;


import dot.codegenerator.ICodeGenerator;
import dot.codegenerator.Instruction;
import dot.parser.nodes.INode;

public class Begin_Generator implements IGenerator {

	ICodeGenerator cg;

	@SuppressWarnings("unchecked")
	public boolean generate(List programList, INode node) throws Exception {
		
		IInstruction beginEnv = new Instruction(IConstants.OPCODES.LOCAL, node);
		programList.add(beginEnv);
		cg.pushEnvironment();
		for (int i = 1; i < node.getChildCount(); i++) {
				cg.generateCode(node.getChild(i));
		}

		if (node.getChildCount() == 1) {
			Instruction constOp = new Instruction(IConstants.OPCODES.CONST, node);
			constOp.pushNil();
			programList.add(constOp);
		}
		/* Return statement */
		IInstruction dropOp = new Instruction(IConstants.OPCODES.DROP, node);
		programList.add(dropOp);
		cg.popEnvironment();
		return false;
	}

	public String getIdentifier() {
		return "begin";
	}

	public void initialize(ICodeGenerator codegen) {
		cg = codegen;
	}

}
