/**
 *  This file is part of
 *  
 *  Solenopsis 2: A Framework for the Development of Ant Algorithms
 *  
 *  Copyright (C) 2006-2009 Amos Brocco <amos.brocco@unifr.ch>
 *  						Pervasive and Artificial Intelligence Research Group
 *  						Department of Informatics DIUF
 *  						University of Fribourg, Fribourg
 *  						Switzerland
 *
 *  Solenopsis 2 is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  Solenopsis 2 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with Solenopsis 2.  If not, see <http://www.gnu.org/licenses/>.
 *  
 */
package dot.codegenerator.modules;


import java.util.List;

import com.fuzzbug.solenopsis.sdk.lib.IConstants;


import logging.Logger;

import dot.codegenerator.ICodeGenerator;
import dot.codegenerator.Instruction;
import dot.parser.nodes.INode;

public class Call_Generator implements IGenerator {

	ICodeGenerator cg;

	@SuppressWarnings("unchecked")
	public boolean generate(List programList, INode node) throws Exception {
		/* Generate code for parameters */
		for (int i = 1; i < node.getChildCount(); i++)
			cg.generateCode(node.getChild(i));
		/* Function call */
		Instruction fcall = null;
		if (cg.isVisibleFunction(node.getChild(0).getString())) {
			fcall = new Instruction(IConstants.OPCODES.ICALL, node);
			fcall.pushParameter(node.getChild(0).getString().trim());
			fcall.pushParameter(node.getChildCount() - 1);
			programList.add(fcall);
		} else if (cg.isVisibleVariable(node.getChild(0).getString())) {
			/* It's a lambda... Load variable on the stack */
			Instruction loadOp = new Instruction(IConstants.OPCODES.LOAD, node);
			String vname = node.getChild(0).getString();
			loadOp.pushParameter(vname);
			programList.add(loadOp);
			/* Manage call */
			fcall = new Instruction(IConstants.OPCODES.LCALL, node);
			fcall.pushParameter(node.getChildCount() - 1);
			programList.add(fcall);
		} else {
			if (node.getChild(0).getString().trim().equals("")) {
				Logger.CompilerFatal(node, cg, "Invalid function name %s", node.getChild(0).getString());
			}
			fcall = new Instruction(IConstants.OPCODES.ECALL, node);
			fcall.pushParameter(cg.getFunctionName(node.getChild(0).getString().trim()));
			fcall.pushParameter(node.getChildCount() - 1);
			programList.add(fcall);
		}

		return false;
	}

	public String getIdentifier() {
		return "++ call ++";
	}

	public void initialize(ICodeGenerator codegen) {
		cg = codegen;
	}

}
