/**
 *  This file is part of
 *  
 *  Solenopsis 2: A Framework for the Development of Ant Algorithms
 *  
 *  Copyright (C) 2006-2009 Amos Brocco <amos.brocco@unifr.ch>
 *  						Pervasive and Artificial Intelligence Research Group
 *  						Department of Informatics DIUF
 *  						University of Fribourg, Fribourg
 *  						Switzerland
 *
 *  Solenopsis 2 is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  Solenopsis 2 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with Solenopsis 2.  If not, see <http://www.gnu.org/licenses/>.
 *  
 */
package dot.codegenerator.modules;


import java.util.List;

import com.fuzzbug.solenopsis.sdk.lib.IConstants;


import dot.codegenerator.ICodeGenerator;
import dot.codegenerator.Instruction;
import dot.parser.nodes.INode;

public class Cond_Generator implements IGenerator {

	ICodeGenerator cg;

	@SuppressWarnings("unchecked")
	public boolean generate(List programList, INode node) throws Exception {
		
		String endJumpLabel = cg.generateJumpTarget();
		for (int i = 1; i < node.getChildCount(); i++) {
			String falseJumpLabel = cg.generateJumpTarget();
			INode condBlock = cg.expand(node.getChild(i));
			INode testBlock = cg.expand(condBlock.getChild(0));
			
			/* Test condition */
			if (!testBlock.getString().equals("else")) {
				cg.generateCode(testBlock);
				/* Jump if false */
				Instruction fjump = new Instruction(IConstants.OPCODES.FJUMP, node);
				fjump.pushParameter(falseJumpLabel);
				programList.add(fjump);
			}
			
			/* Generate true code */
			for (int k = 1; k < condBlock.getChildCount(); k++) {
					cg.generateCode(condBlock.getChild(k));
			}

			/* Jump to global cond end */
			Instruction ejump = new Instruction(IConstants.OPCODES.JUMP, node);
			ejump.pushParameter(endJumpLabel);
			programList.add(ejump);
			/* False label */
			Instruction fLabel = new Instruction(IConstants.OPCODES.LABEL, node);
			fLabel.pushParameter(falseJumpLabel);
			programList.add(fLabel);
		}
		/* Global end label */
		Instruction geLabel = new Instruction(IConstants.OPCODES.LABEL, node);
		geLabel.pushParameter(endJumpLabel);
		programList.add(geLabel);
		
		return false;
	}

	public String getIdentifier() {
		return "cond";
	}

	public void initialize(ICodeGenerator codegen) {
		cg = codegen;
	}

}
