/**
 *  This file is part of
 *  
 *  Solenopsis 2: A Framework for the Development of Ant Algorithms
 *  
 *  Copyright (C) 2006-2009 Amos Brocco <amos.brocco@unifr.ch>
 *  						Pervasive and Artificial Intelligence Research Group
 *  						Department of Informatics DIUF
 *  						University of Fribourg, Fribourg
 *  						Switzerland
 *
 *  Solenopsis 2 is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  Solenopsis 2 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with Solenopsis 2.  If not, see <http://www.gnu.org/licenses/>.
 *  
 */
package dot.codegenerator.modules;


import java.util.List;

import com.fuzzbug.solenopsis.sdk.lib.IConstants;
import com.fuzzbug.solenopsis.sdk.lib.IInstruction;


import logging.Logger;

import dot.codegenerator.ICodeGenerator;
import dot.codegenerator.Instruction;
import dot.parser.nodes.DotNode;
import dot.parser.nodes.INode;
import dot.parser.nodes.ProgramNode;

public class Defun_Generator implements IGenerator {

	ICodeGenerator cg;

	@SuppressWarnings("unchecked")
	public boolean generate(List programList, INode node) throws Exception {
		if(!(node.getParent()  instanceof  ProgramNode)) {
			Logger.CompilerFatal(node, cg, "Cannot define a function inside a block");
		}
		/* Function frame */
		Instruction functionLabel = new Instruction(IConstants.OPCODES.FRAME, node);
		String functionName = node.getChild(1).getChild(0).getString();
		
		if (cg.isGlobalFunction(functionName))
			Logger.CompilerFatal(node, cg, "Duplicate function definition '%s'", functionName);
		cg.pushGlobalFunction(functionName);
		functionLabel.pushParameter(functionName);
		programList.add(functionLabel);
		cg.pushEnvironment();
		cg.pushFunctionBlock(functionName);

		String valist = null;
		/* Parameters binding (if necessary) */
		if (node.getChild(1).getChildCount() > 1) {
			Instruction bindOp = new Instruction(IConstants.OPCODES.BIND, node);
			bindOp.openList();
			for (int x = 1; x < node.getChild(1).getChildCount(); x++) {
				if (node.getChild(1).getChild(x) instanceof DotNode) {
					if (node.getChild(1).getChild(x+1) instanceof DotNode) {
						if (node.getChild(1).getChild(x+2) instanceof DotNode) {
							valist = node.getChild(1).getChild(x+3).getString();
							break;
						}
					}
					Logger.CompilerFatal(node, cg, "Unterminated valist '%s' in function '%s'", node.getChild(1).getChild(x).getString(), functionName);
				}
				
				bindOp.pushParameter(node.getChild(1).getChild(x).getString());
				if (!cg.isLocalVariable(node.getChild(1).getChild(x).getString()))
					cg.pushLocalVariable(node.getChild(1).getChild(x).getString());
				else {
					Logger.CompilerFatal(node, cg, "Duplicate parameter '%s' in function '%s'", node.getChild(1).getChild(x).getString(), functionName);
					//cg.pushLocalVariable(node.getChild(1).getChild(x).getString());
				}
			}
			bindOp.closeList();
			programList.add(bindOp);
			
			if (valist != null) {
				Instruction listOp = new Instruction(IConstants.OPCODES.LIST, node);
				listOp.pushParameter(-1);
				programList.add(listOp);
				Instruction valistbindOp = new Instruction(IConstants.OPCODES.BIND, node);
				valistbindOp.openList();
				valistbindOp.pushParameter(valist);
				valistbindOp.closeList();
				programList.add(valistbindOp);
				cg.pushLocalVariable(valist);
			}
		}
		
		/* Generate code */
		cg.generateCode(node.getChild(2));
		/* Return statement */
		IInstruction returnOp = new Instruction(IConstants.OPCODES.RETURN, node);
		programList.add(returnOp);
		cg.popEnvironment();
		cg.popFunctionBlock();
		return false;
	}

	public String getIdentifier() {
		return "defun";
	}

	public void initialize(ICodeGenerator codegen) {
		cg = codegen;
	}

}
