/**
 *  This file is part of
 *  
 *  Solenopsis 2: A Framework for the Development of Ant Algorithms
 *  
 *  Copyright (C) 2006-2009 Amos Brocco <amos.brocco@unifr.ch>
 *  						Pervasive and Artificial Intelligence Research Group
 *  						Department of Informatics DIUF
 *  						University of Fribourg, Fribourg
 *  						Switzerland
 *
 *  Solenopsis 2 is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  Solenopsis 2 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with Solenopsis 2.  If not, see <http://www.gnu.org/licenses/>.
 *  
 */
package dot.codegenerator.modules;


import java.util.List;

import com.fuzzbug.solenopsis.sdk.lib.IConstants;
import com.fuzzbug.solenopsis.sdk.lib.IInstruction;


import logging.Logger;

import dot.codegenerator.ICodeGenerator;
import dot.codegenerator.Instruction;
import dot.parser.nodes.INode;
import dot.parser.nodes.ProgramNode;
import dot.parser.nodes.SymbolNode;

public class Defvar_Generator implements IGenerator {

	ICodeGenerator cg;

	@SuppressWarnings("unchecked")
	public boolean generate(List programList, INode node) throws Exception {
		
		
		if(!(node.getParent()  instanceof  ProgramNode)) {
			// Not very clean :)

			cg.pushLocalVariable(node.getChild(1).getString());
			/* Generate code for the value */
			cg.generateCode(node.getChild(2));
			
			/* Duplicate the value on the stack */
			IInstruction dup = new Instruction(IConstants.OPCODES.DUP, node);
			programList.add(dup);
			
			/* Save the value in the variable */
			Instruction store = new Instruction(IConstants.OPCODES.STORE, node);
			store.pushParameter(node.getChild(1).getString());
			programList.add(store);
		} else {

			if (!(node.getChild(1) instanceof SymbolNode)) {
				Logger.CompilerFatal(node, cg, "Invalid variable name '%s'", node.getChild(1).getString());
			}
			
			if (cg.isGlobalVariable(node.getChild(1).getString())) {
				Logger.CompilerFatal(node, cg, "Duplicate variable name '%s'", node.getChild(1).getString());
			}
			cg.pushGlobalVariable(node.getChild(1).getString());
			/* Generate code for the value */
			cg.generateCode(node.getChild(2));
			
			/* Duplicate the value on the stack */
			IInstruction dup = new Instruction(IConstants.OPCODES.DUP, node);
			programList.add(dup);
			
			/* Save the value in the variable */
			Instruction store = new Instruction(IConstants.OPCODES.GSTORE, node);
			store.pushParameter(node.getChild(1).getString());
			programList.add(store);
			
		}
		return false;
	}

	public String getIdentifier() {
		return "defvar";
	}

	public void initialize(ICodeGenerator codegen) {
		cg = codegen;
	}

}
