/**
 *  This file is part of
 *  
 *  Solenopsis 2: A Framework for the Development of Ant Algorithms
 *  
 *  Copyright (C) 2006-2009 Amos Brocco <amos.brocco@unifr.ch>
 *  						Pervasive and Artificial Intelligence Research Group
 *  						Department of Informatics DIUF
 *  						University of Fribourg, Fribourg
 *  						Switzerland
 *
 *  Solenopsis 2 is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  Solenopsis 2 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with Solenopsis 2.  If not, see <http://www.gnu.org/licenses/>.
 *  
 */
package dot.codegenerator.modules;


import java.util.List;

import com.fuzzbug.solenopsis.sdk.lib.IConstants;
import com.fuzzbug.solenopsis.sdk.lib.IInstruction;


import dot.codegenerator.ICodeGenerator;
import dot.codegenerator.Instruction;
import dot.parser.nodes.INode;

public class For_Generator implements IGenerator {

	ICodeGenerator cg;

	@SuppressWarnings("unchecked")
	public boolean generate(List programList, INode node) throws Exception {
		String forExitTag = cg.generateJumpTarget();
		String forStartTag = cg.generateJumpTarget();

		IInstruction forCall;
		cg.pushLoopBreakTarget(forExitTag);
		cg.pushLoopContinueTarget(forStartTag);
		/* Create local for environment */
		forCall = new Instruction(IConstants.OPCODES.LOCAL, node);
		programList.add(forCall);
		/* Initialize variable */
		cg.pushEnvironment();
		generateDefinition(programList, node.getChild(1));
		/* For start label */
		Instruction sLabel = new Instruction(IConstants.OPCODES.LABEL, node);
		sLabel.pushParameter(forStartTag);
		programList.add(sLabel);
		/* Generate code for the test value */
		cg.generateCode(node.getChild(2));
		/* Test the condition */
		Instruction fJump = new Instruction(IConstants.OPCODES.FJUMP, node);
		fJump.pushParameter(forExitTag);
		programList.add(fJump);
		/* Generate code */
		cg.generateCode(node.getChild(4));
		/* Generate increment */
		cg.generateCode(node.getChild(3));
		/* Jump to start */
		Instruction sJump = new Instruction(IConstants.OPCODES.JUMP, node);
		sJump.pushParameter(forStartTag);
		programList.add(sJump);
		/* Exit label */
		Instruction eLabel = new Instruction(IConstants.OPCODES.LABEL, node);
		eLabel.pushParameter(forExitTag);
		programList.add(eLabel);
		/* Drop the local environment */
		IInstruction dropOp = new Instruction(IConstants.OPCODES.DROP, node);
		programList.add(dropOp);
		/* Push something on the stack */
		Instruction nil = new Instruction(IConstants.OPCODES.CONST, node);
		nil.pushNil();
		programList.add(nil);
		cg.popLoopBreakTarget();
		cg.popLoopContinueTarget();
		cg.popEnvironment();
		return false;
	}

	@SuppressWarnings("unchecked")
	private void generateDefinition(List programList, INode node) throws Exception {
		/* Generate code for the value */
		cg.generateCode(node.getChild(1));
		/* Save the value in the variable */
		Instruction store = new Instruction(IConstants.OPCODES.STORE, node);
		store.pushParameter(node.getChild(0).getString());
		cg.pushLocalVariable(node.getChild(0).getString());
		programList.add(store);
	}

	public String getIdentifier() {
		return "for";
	}

	public void initialize(ICodeGenerator codegen) {
		cg = codegen;
	}

}
