/**
 *  This file is part of
 *  
 *  Solenopsis 2: A Framework for the Development of Ant Algorithms
 *  
 *  Copyright (C) 2006-2009 Amos Brocco <amos.brocco@unifr.ch>
 *  						Pervasive and Artificial Intelligence Research Group
 *  						Department of Informatics DIUF
 *  						University of Fribourg, Fribourg
 *  						Switzerland
 *
 *  Solenopsis 2 is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  Solenopsis 2 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with Solenopsis 2.  If not, see <http://www.gnu.org/licenses/>.
 *  
 */
package dot.codegenerator.modules;


import java.util.List;

import com.fuzzbug.solenopsis.sdk.lib.IConstants;


import dot.codegenerator.ICodeGenerator;
import dot.codegenerator.Instruction;
import dot.parser.nodes.INode;

public class If_Generator implements IGenerator {

	ICodeGenerator cg;

	@SuppressWarnings("unchecked")
	public boolean generate(List programList, INode node) throws Exception {

		String falseJumpLabel = cg.generateJumpTarget();
		String endJumpLabel = cg.generateJumpTarget();
		/* Generate test code */
		cg.generateCode(node.getChild(1));
		
		if (node.getChildCount() > 3) {
			/* Jump if false */
			Instruction fjump = new Instruction(IConstants.OPCODES.FJUMP, node);
			fjump.pushParameter(falseJumpLabel);
			programList.add(fjump);
			/* Generate code for true condition */
			cg.generateCode(node.getChild(2));
			/* Jump to end */
			Instruction ejump = new Instruction(IConstants.OPCODES.JUMP, node);
			ejump.pushParameter(endJumpLabel);
			programList.add(ejump);
			/* False label */
			Instruction fLabel = new Instruction(IConstants.OPCODES.LABEL, node);
			fLabel.pushParameter(falseJumpLabel);
			programList.add(fLabel);
			/* Generate code for false condition */
			int falseBlockIndex = 3;
			/* If there is an "else" the false block is the next */
			if (node.getChild(3).getString().equals("else"))
				falseBlockIndex = 4;
			/* If there are other block parse it */
			if (node.getChildCount() > falseBlockIndex) {
				cg.generateCode(node.getChild(falseBlockIndex));
			}
		} else {
			/* Jump if false */
			Instruction fjump = new Instruction(IConstants.OPCODES.FJUMP, node);
			fjump.pushParameter(falseJumpLabel);
			programList.add(fjump);

			/* Generate code for true condition */
			cg.generateCode(node.getChild(2));
			
			/* Jump to end */
			Instruction ejump = new Instruction(IConstants.OPCODES.JUMP, node);
			ejump.pushParameter(endJumpLabel);
			programList.add(ejump);
			/* False label */
			Instruction fLabel = new Instruction(IConstants.OPCODES.LABEL, node);
			fLabel.pushParameter(falseJumpLabel);
			programList.add(fLabel);
		}
		/* End label */
		Instruction eLabel = new Instruction(IConstants.OPCODES.LABEL, node);
		eLabel.pushParameter(endJumpLabel);
		programList.add(eLabel);
		return false;
	}

	public String getIdentifier() {
		return "if";
	}

	public void initialize(ICodeGenerator codegen) {
		cg = codegen;
	}

}
