/**
 *  This file is part of
 *  
 *  Solenopsis 2: A Framework for the Development of Ant Algorithms
 *  
 *  Copyright (C) 2006-2009 Amos Brocco <amos.brocco@unifr.ch>
 *  						Pervasive and Artificial Intelligence Research Group
 *  						Department of Informatics DIUF
 *  						University of Fribourg, Fribourg
 *  						Switzerland
 *
 *  Solenopsis 2 is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  Solenopsis 2 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with Solenopsis 2.  If not, see <http://www.gnu.org/licenses/>.
 *  
 */
package dot.codegenerator.modules;


import java.util.List;

import com.fuzzbug.solenopsis.sdk.lib.IConstants;


import logging.Logger;

import dot.codegenerator.ICodeGenerator;
import dot.codegenerator.Instruction;
import dot.parser.nodes.INode;

public class Label_Generator implements IGenerator {

	ICodeGenerator cg;

	@SuppressWarnings("unchecked")
	public boolean generate(List programList, INode node) throws Exception {
		if (node.getChildCount() != 2) {
			Logger.CompilerFatal(node, cg, "Label not specified");
		}
		/* Jump */
		String labname = cg.getCurrentFunctionBlock()+"@@"+node.getChild(1).getString();
		if (cg.isValidLabel(labname)) {
			Instruction eLabel = new Instruction(IConstants.OPCODES.LABEL, node);
			eLabel.pushParameter(labname);
			programList.add(eLabel);
		} else {
			Logger.CompilerFatal(node, cg, "Label '%s' already defined", labname);
		}
		return false;
	}

	public String getIdentifier() {
		return "label";
	}

	public void initialize(ICodeGenerator codegen) {
		cg = codegen;
	}

}
