/**
 *  This file is part of
 *  
 *  Solenopsis 2: A Framework for the Development of Ant Algorithms
 *  
 *  Copyright (C) 2006-2009 Amos Brocco <amos.brocco@unifr.ch>
 *  						Pervasive and Artificial Intelligence Research Group
 *  						Department of Informatics DIUF
 *  						University of Fribourg, Fribourg
 *  						Switzerland
 *
 *  Solenopsis 2 is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  Solenopsis 2 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with Solenopsis 2.  If not, see <http://www.gnu.org/licenses/>.
 *  
 */
package dot.codegenerator.modules;


import java.util.ArrayList;
import java.util.List;

import com.fuzzbug.solenopsis.sdk.lib.IConstants;
import com.fuzzbug.solenopsis.sdk.lib.IInstruction;


import dot.codegenerator.ICodeGenerator;
import dot.codegenerator.Instruction;
import dot.parser.nodes.INode;

public class Lambda_Generator implements IGenerator {

	ICodeGenerator cg;

	@SuppressWarnings("unchecked")
	public boolean generate(List programList, INode node) throws Exception {
		
		ArrayList bindParam = new ArrayList();
		String lambdaDefTarget = cg.generateJumpTarget();
		String lambdaStartTarget = cg.generateJumpTarget();
		/* Jump to lambda instruction */
		Instruction ejump = new Instruction(IConstants.OPCODES.JUMP, node);
		ejump.pushParameter(lambdaDefTarget);
		programList.add(ejump);
		Instruction lsLabel = new Instruction(IConstants.OPCODES.LABEL, node);
		lsLabel.pushParameter(lambdaStartTarget);
		programList.add(lsLabel);
		cg.pushEnvironment();
		/* Parameters binding (if necessary) */
		if (node.getChild(1).getChildCount() > 0) {
			Instruction bindOp = new Instruction(IConstants.OPCODES.BIND, node);
			bindOp.openList();
			for (int x = 0; x < node.getChild(1).getChildCount(); x++) {
				bindOp.pushParameter(node.getChild(1).getChild(x).getString());
				bindParam.add(node.getChild(1).getChild(x).getString());
				cg.pushLocalVariable(node.getChild(1).getChild(x).getString());
			}
			bindOp.closeList();
			programList.add(bindOp);
		}
		/* Generate code for body */
		cg.pushLambdaFrame();
		cg.generateCode(node.getChild(2));
		/* Return statement */
		IInstruction returnOp = new Instruction(IConstants.OPCODES.RETURN, node);
		programList.add(returnOp);
		/* Lambda label */
		Instruction fLabel = new Instruction(IConstants.OPCODES.LABEL, node);
		fLabel.pushParameter(lambdaDefTarget);
		programList.add(fLabel);
		/* Lambda instruction */
		Instruction lambdaOp = new Instruction(IConstants.OPCODES.LAMBDA, node);
		lambdaOp.pushParameter(lambdaStartTarget);
		lambdaOp.openList();
		List bVar = cg.getLambdaVars();
		for(int i = 0; i < bVar.size(); i++) {
			if(!bindParam.contains((String) bVar.get(i))
					&& !cg.isGlobalVariable((String) bVar.get(i)))
				lambdaOp.pushParameter((String) bVar.get(i));
		}
		lambdaOp.closeList();
		programList.add(lambdaOp);
		cg.popEnvironment();
		return false;
	}

	public String getIdentifier() {
		return "lambda";
	}

	public void initialize(ICodeGenerator codegen) {
		cg = codegen;
	}

}
