/**
 *  This file is part of
 *  
 *  Solenopsis 2: A Framework for the Development of Ant Algorithms
 *  
 *  Copyright (C) 2006-2009 Amos Brocco <amos.brocco@unifr.ch>
 *  						Pervasive and Artificial Intelligence Research Group
 *  						Department of Informatics DIUF
 *  						University of Fribourg, Fribourg
 *  						Switzerland
 *
 *  Solenopsis 2 is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  Solenopsis 2 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with Solenopsis 2.  If not, see <http://www.gnu.org/licenses/>.
 *  
 */
package dot.codegenerator.modules;


import java.util.List;

import com.fuzzbug.solenopsis.sdk.lib.IConstants;
import com.fuzzbug.solenopsis.sdk.lib.IInstruction;


import logging.Logger;

import dot.codegenerator.ICodeGenerator;
import dot.codegenerator.Instruction;
import dot.parser.nodes.INode;

public class Let_Generator implements IGenerator {

	ICodeGenerator cg;

	int variableCount = 0;

	@SuppressWarnings("unchecked")
	public boolean generate(List programList, INode node) throws Exception {
		IInstruction letEnv = new Instruction(IConstants.OPCODES.LOCAL, node);
		programList.add(letEnv);
		INode definitions = cg.expand(node.getChild(1));
		cg.pushEnvironment();
		for (int k = 0; k < definitions.getChildCount(); k++)
			generateDefinition(programList, cg.expand(definitions.getChild(k)));

		for (int i = 2; i < node.getChildCount(); i++) {
			cg.generateCode(node.getChild(i));
		}
		/* Return statement */
		IInstruction dropOp = new Instruction(IConstants.OPCODES.DROP, node);
		programList.add(dropOp);
		cg.popEnvironment();
		
		return false;
	}

	@SuppressWarnings("unchecked")
	private void generateDefinition(List programList, INode node) throws Exception {
		if (node.getChildCount() != 2) {
			Logger.CompilerFatal(node, cg, "Malformed let definition");
		}

		/* Generate code for the value */
		cg.generateCode(node.getChild(1));
		/* Save the value in the variable */
		Instruction store = new Instruction(IConstants.OPCODES.STORE, node);
		if (!cg.isLocalVariable(node.getChild(0).getString())) {
			cg.pushLocalVariable(node.getChild(0).getString());
		} else {
			Logger.CompilerFatal(node, cg, "Duplicate variable definition '%s'", node.getChild(0).getString());
			//cg.pushLocalVariable(node.getChild(0).getString());
		}
		store.pushParameter(node.getChild(0).getString());
		programList.add(store);
	}

	public String getIdentifier() {
		return "let";
	}

	public void initialize(ICodeGenerator codegen) {
		cg = codegen;
	}

}
