/**
 *  This file is part of
 *  
 *  Solenopsis 2: A Framework for the Development of Ant Algorithms
 *  
 *  Copyright (C) 2006-2009 Amos Brocco <amos.brocco@unifr.ch>
 *  						Pervasive and Artificial Intelligence Research Group
 *  						Department of Informatics DIUF
 *  						University of Fribourg, Fribourg
 *  						Switzerland
 *
 *  Solenopsis 2 is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  Solenopsis 2 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with Solenopsis 2.  If not, see <http://www.gnu.org/licenses/>.
 *  
 */
package dot.codegenerator.modules;


import java.util.List;

import com.fuzzbug.solenopsis.sdk.lib.IConstants;


import logging.Logger;

import dot.codegenerator.ICodeGenerator;
import dot.codegenerator.Instruction;
import dot.parser.nodes.INode;

public class Set_Generator implements IGenerator {

	ICodeGenerator cg;

	@SuppressWarnings("unchecked")
	public boolean generate(List programList, INode node) throws Exception {
		
		/* Generate code for the value */
		cg.generateCode(node.getChild(2));

		/* Duplicate TOS */
		Instruction dup = new Instruction(IConstants.OPCODES.DUP, node);
		dup.pushNil();
		programList.add(dup);
		/* Save the value in the variable */
		Instruction store = null;
		String vname = node.getChild(1).getString();
		if(cg.isGlobalVariable(vname))
			store = new Instruction(IConstants.OPCODES.GSET, node);
		else
			store = new Instruction(IConstants.OPCODES.SET, node);
		if (!cg.isVisibleVariable(vname)) {
			Logger.CompilerFatal(node, cg, "Undefined variable '%s'", vname);
		}
		store.pushParameter(vname);
		programList.add(store);
		/* The value is on the stack */
		return false;
	}

	public String getIdentifier() {
		return "set!";
	}

	public void initialize(ICodeGenerator codegen) {
		cg = codegen;
	}

}
