/**
 *  This file is part of
 *  
 *  Solenopsis 2: A Framework for the Development of Ant Algorithms
 *  
 *  Copyright (C) 2006-2009 Amos Brocco <amos.brocco@unifr.ch>
 *  						Pervasive and Artificial Intelligence Research Group
 *  						Department of Informatics DIUF
 *  						University of Fribourg, Fribourg
 *  						Switzerland
 *
 *  Solenopsis 2 is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  Solenopsis 2 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with Solenopsis 2.  If not, see <http://www.gnu.org/licenses/>.
 *  
 */
package dot.codegenerator.optimizers;


import java.util.ArrayList;
import java.util.Hashtable;
import java.util.List;

import com.fuzzbug.solenopsis.sdk.lib.IConstants;
import com.fuzzbug.solenopsis.sdk.lib.IInstruction;


import logging.Logger;
import dot.codegenerator.ICodeGenerator;
import dot.codegenerator.Instruction;
import dot.parser.nodes.INode;

public class CreateRelativeJumpsProcessor implements IOptimizer {

	@SuppressWarnings("unchecked")
	private Hashtable jumpTargets;

	@SuppressWarnings("unchecked")
	public List optimize(List programList, ICodeGenerator cg) throws Exception {
		jumpTargets = new Hashtable();

		/* Retrieve jump targets positions */
		ArrayList tmpList = new ArrayList();
		for (int i = 0; i < programList.size(); i++) {
			IInstruction instr = (IInstruction) programList.get(i);
			if ((instr.getOpCode() == IConstants.OPCODES.LABEL)
					|| (instr.getOpCode() == IConstants.OPCODES.FRAME)) {
				jumpTargets.put((String) instr.getParameters().get(0),
						new Integer(tmpList.size()));
			} else
				tmpList.add(instr);
		}

		programList = tmpList;

		for (int i = 0; i < programList.size(); i++) {
			Instruction instr = (Instruction) programList.get(i);
			if ((instr.getOpCode() == IConstants.OPCODES.JUMP)
					|| (instr.getOpCode() == IConstants.OPCODES.FJUMP)
					|| (instr.getOpCode() == IConstants.OPCODES.LAMBDA)
					|| (instr.getOpCode() == IConstants.OPCODES.ICALL)) {
				String ident = (String) instr.getParameters().remove(0);
				instr.getParameters().add(0,
						new Integer(getRelativeJump(instr.getNode(), ident, i)));
			}
		}
		
		for (int i = 0; i < programList.size(); i++) {
			Instruction instr = (Instruction) programList.get(i);
			if ((instr.getOpCode() == IConstants.OPCODES.PC)) {
				instr.setOpCode(IConstants.OPCODES.CONST);
				String ident = (String) instr.getParameters().remove(0);
				instr.getParameters().add(0,
						new Integer(getJumpTargetPosition(instr.getNode(), ident)));
			}
		}
		
		return programList;
	}

	private int getRelativeJump(INode node, String label, int currentPosition) throws Exception {
		int target = getJumpTargetPosition(node, label);
		return target - currentPosition;
	}
	
	private int getJumpTargetPosition(INode node, String label) throws Exception {
		if(!jumpTargets.containsKey(label)) {
			Logger.CompilerFatal(node, null, "Label '%s' not defined", label);
			return -1;
		}
		return ((Integer) jumpTargets.get(label)).intValue();
	}

}
