/**
 *  This file is part of
 *  
 *  Solenopsis 2: A Framework for the Development of Ant Algorithms
 *  
 *  Copyright (C) 2006-2009 Amos Brocco <amos.brocco@unifr.ch>
 *  						Pervasive and Artificial Intelligence Research Group
 *  						Department of Informatics DIUF
 *  						University of Fribourg, Fribourg
 *  						Switzerland
 *
 *  Solenopsis 2 is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  Solenopsis 2 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with Solenopsis 2.  If not, see <http://www.gnu.org/licenses/>.
 *  
 */
package dot.codegenerator.optimizers;


import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;

import com.fuzzbug.solenopsis.sdk.lib.IConstants;
import com.fuzzbug.solenopsis.sdk.lib.IInstruction;


import dot.codegenerator.ICodeGenerator;

public class ShortenNamesProcessor implements IOptimizer {

	@SuppressWarnings("unchecked")
	private Hashtable varReferences;

	private int currentVarName;

	@SuppressWarnings("unchecked")
	public List optimize(List programList, ICodeGenerator cg) {
		varReferences = new Hashtable();
		class VarUsageComparator implements Comparator {
			private Hashtable varusagetable;

			public VarUsageComparator(Hashtable varusage) {
				varusagetable = varusage;
			}

			public int compare(Object item1, Object item2) {
				// Get the value of the properties
				int value1 = ((Integer) varusagetable.get((String) item1))
						.intValue();
				int value2 = ((Integer) varusagetable.get((String) item2))
						.intValue();
				if (value1 > value2)
					return -1;
				else if (value1 == value2)
					return 0;
				else
					return 1;
			}
		}

		varReferences = new Hashtable();
		ArrayList varNames = new ArrayList();
		for (int i = 0; i < programList.size(); i++) {
			IInstruction instr = (IInstruction) programList.get(i);
			if ((instr.getOpCode() == IConstants.OPCODES.LOAD)
					|| (instr.getOpCode() == IConstants.OPCODES.STORE)
					|| (instr.getOpCode() == IConstants.OPCODES.SET)
					|| (instr.getOpCode() == IConstants.OPCODES.GSTORE)
					|| (instr.getOpCode() == IConstants.OPCODES.GSET)
					|| (instr.getOpCode() == IConstants.OPCODES.GLOAD)) {
				incrementVariableUsage((String) instr.getParameters().get(0));
				if (!(varNames.contains((String) instr.getParameters().get(0))))
					varNames.add((String) instr.getParameters().get(0));
			} else if ((instr.getOpCode() == IConstants.OPCODES.BIND)) {
				ArrayList bindings = (ArrayList) instr.getParameters().get(0);
				for (int t = 0; t < bindings.size(); t++) {
					incrementVariableUsage((String) bindings.get(t));
				}
			} else if ((instr.getOpCode() == IConstants.OPCODES.LAMBDA)) {
				ArrayList bindings = (ArrayList) instr.getParameters().get(1);
				for (int t = 0; t < bindings.size(); t++) {
					incrementVariableUsage((String) bindings.get(t));
				}
			}
		}
		Hashtable varSubstitutions = new Hashtable();
		Comparator cmp = new VarUsageComparator(varReferences);
		Collections.sort(varNames, cmp);
		Iterator iter = varNames.iterator();
		while (iter.hasNext()) {
			String var = (String) iter.next();
			// Do not replace variables starting with $
			if (var.charAt(0) == '$')
				varSubstitutions.put(var, var);
			else
				varSubstitutions.put(var, getNextIdentifier());
		}

		for (int i = 0; i < programList.size(); i++) {
			IInstruction instr = (IInstruction) programList.get(i);
			if ((instr.getOpCode() == IConstants.OPCODES.LOAD)
					|| (instr.getOpCode() == IConstants.OPCODES.STORE)
					|| (instr.getOpCode() == IConstants.OPCODES.SET)
					|| (instr.getOpCode() == IConstants.OPCODES.GSTORE)
					|| (instr.getOpCode() == IConstants.OPCODES.GSET)
					|| (instr.getOpCode() == IConstants.OPCODES.GLOAD)) {
				String varname = (String) instr.getParameters().remove(0);
				instr.getParameters().add(0, varSubstitutions.get(varname));
			} else if ((instr.getOpCode() == IConstants.OPCODES.BIND)) {
				ArrayList bindings = (ArrayList) instr.getParameters().get(0);
				for (int t = 0; t < bindings.size(); t++) {
					String varname = (String) bindings.remove(0);
					bindings.add(varSubstitutions.get(varname));
				}
			} else if ((instr.getOpCode() == IConstants.OPCODES.LAMBDA)) {
				ArrayList bindings = (ArrayList) instr.getParameters().get(1);
				for (int t = 0; t < bindings.size(); t++) {
					String varname = (String) bindings.remove(0);
					bindings.add(varSubstitutions.get(varname));
				}
			}
		}
		return programList;
	}

	@SuppressWarnings("unchecked")
	private void incrementVariableUsage(String varname) {
		if (varReferences.containsKey(varname)) {
			int count = ((Integer) varReferences.get(varname)).intValue();
			varReferences.remove(varname);
			varReferences.put(varname, new Integer(count + 1));
		} else
			varReferences.put(varname, new Integer(1));
	}

	public String getNextIdentifier() {
		currentVarName++;
		return Integer.toString(currentVarName, 36);
	}

}
