/**
 *  This file is part of
 *  
 *  Solenopsis 2: A Framework for the Development of Ant Algorithms
 *  
 *  Copyright (C) 2006-2009 Amos Brocco <amos.brocco@unifr.ch>
 *  						Pervasive and Artificial Intelligence Research Group
 *  						Department of Informatics DIUF
 *  						University of Fribourg, Fribourg
 *  						Switzerland
 *
 *  Solenopsis 2 is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  Solenopsis 2 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with Solenopsis 2.  If not, see <http://www.gnu.org/licenses/>.
 *  
 */
package dot.compiler;

import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;

import com.fuzzbug.solenopsis.sdk.lib.IInstruction;

import logging.Logger;
import providers.IResourceProvider;
import dot.codegenerator.CodeGenerator;
import dot.lexer.Lexer;
import dot.parser.Parser;
import dot.parser.nodes.INode;
import dot.preprocessor.Preprocessor;

public class Compiler {

	public static List<IInstruction> compile(IResourceProvider rp, String name, InputStream osource) throws Exception {

		/* Preprocess */
		List<String> code = new ArrayList<String>();
		InputStream dslis = Preprocessor.class.getClassLoader().getResourceAsStream("dot/preprocessor/dsl.defs");
		int dsloffset = Preprocessor.preprocess(rp, dslis, osource, code);
		
		/* Create a lexer object */
		Lexer lex = null;
		try {
			lex = new Lexer(name, code, dsloffset);
		} catch (IOException e) {
			Logger.fatal("Error reading from input stream:\n"
					+ e.getMessage());
		}
		/* Create a parser object and pass the lexer */
		Parser par = null;
		try {
			par = new Parser(lex);
		} catch (IOException e) {
			Logger.fatal("Error reading from input stream:\n"
					+ e.getMessage());
		}
		/* Open the output file */
		INode root = null;
		root = par.getParseTree();
		CodeGenerator cg = new CodeGenerator(root);
		return cg.generate();
	}

}
