/**
 *  This file is part of
 *  
 *  Solenopsis 2: A Framework for the Development of Ant Algorithms
 *  
 *  Copyright (C) 2006-2009 Amos Brocco <amos.brocco@unifr.ch>
 *  						Pervasive and Artificial Intelligence Research Group
 *  						Department of Informatics DIUF
 *  						University of Fribourg, Fribourg
 *  						Switzerland
 *
 *  Solenopsis 2 is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  Solenopsis 2 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with Solenopsis 2.  If not, see <http://www.gnu.org/licenses/>.
 *  
 */
package dot.lexer;

public class Token {

	private int type;

	private String stringValue = "";

	private Number numberValue = 0;

	private int line;
	
	private String filename;

	private int position;

	public Token(int tokentype, String f, int l, int pos) {
		type = tokentype;
		line = l;
		position = pos;
		filename = f;
	}

	public Token(int tokentype, String value, String f,  int l, int pos) {
		type = tokentype;
		stringValue = value;
		line = l;
		position = pos;
		filename = f;
	}

	public Token(int tokentype, Number value, String f,  int l, int pos) {
		type = tokentype;
		line = l;
		position = pos;
		numberValue = value;
		filename = f;
	}

	
	public String toString() {
		return this.getClass().getName();
	}

	public int getType() {
		return type;
	}

	public String getString() {
		return stringValue;
	}

	public Number getNumber() {
		return numberValue;
	}

	public int getLine() {
		return line;
	}
	
	public String getFilename() {
		return filename;
	}

	public int getPosition() {
		return position;
	}
	
	public Token clone() {
		if (type == Constants.STRING) {
			return new Token(type, new String(stringValue), filename, line, position);
		} else if (type == Constants.SYMBOL) {
			return new Token(type, new String(stringValue), filename, line, position);
		} else if (type == Constants.INTNUMBER) {
			return new Token(type, numberValue, filename, line, position);
		} else if (type == Constants.FLOATNUMBER) {
			return new Token(type, numberValue, filename, line, position);
		} else if (type == Constants.LONGNUMBER) {
			return new Token(type, numberValue, filename, line, position);
		} else if (type == Constants.DOUBLENUMBER) {
			return new Token(type, numberValue, filename, line, position);
		} else
			return new Token(type, filename, line, position);
	}

	public void setString(String s) {
		stringValue = new String(s);
	}

}
