/**
 *  This file is part of
 *  
 *  Solenopsis 2: A Framework for the Development of Ant Algorithms
 *  
 *  Copyright (C) 2006-2009 Amos Brocco <amos.brocco@unifr.ch>
 *  						Pervasive and Artificial Intelligence Research Group
 *  						Department of Informatics DIUF
 *  						University of Fribourg, Fribourg
 *  						Switzerland
 *
 *  Solenopsis 2 is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  Solenopsis 2 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with Solenopsis 2.  If not, see <http://www.gnu.org/licenses/>.
 *  
 */
package dot.parser.nodes;

import java.util.ArrayList;

import logging.Logger;

import dot.lexer.Token;


public abstract class Node implements INode {

	protected ArrayList<INode> children;

	protected Token token;

	private int line;

	private int position;
	
	private INode parent;
	
	protected String code;
	


	/*public String toString() {
		return super.toString()+" @ "+code;
	}*/
	
	public Node(Token tok, String code) {
		token = tok;
		children = new ArrayList<INode>();
		line = tok.getLine();
		position = tok.getPosition();
		this.code = code;	
	}

	public String getFilename() {
		return token.getFilename();
	}
	
	public void addChild(INode child) {
		children.add(child);
		child.setParent(this);
	}
	
	public ArrayList<INode> getChildren() {
		return children;
	}

	public INode getChild(int index) throws Exception {
		try {
			return (INode) children.get(index);
		} catch (Exception e) {
			Logger.CompilerFatal(this, null, "Malformed expression: wrong number of parameters");
			return null;
		}
	}

	public int getChildCount() {
		return children.size();
	}

	public int getType() {
		return token.getType();
	}

	public String getString() {
		return token.getString();
	}
	

	public void setString(String string) {
		token.setString(string);
	}


	public Number getNumber() {
		return token.getNumber();
	}

	public int getLine() {
		return line;
	}
	
	public void replaceChild(int index, INode newchild) {
		children.add(index, newchild);
		children.remove(index + 1);
	}
	
	public void addChildAt(int index, INode child) {
		children.add(index, child);
	}
	
	public void removeChild(int index) {
		children.remove(index);
	}

	public int getPosition() {
		return position;
	}
	
	public void setParent(INode parent) {
		this.parent = parent;
	}
	
	public INode getParent() {
		return parent;
	}
	
	public void setCode(String code) {
		this.code = code;
	}
	
	public String getCode() {
		return code;
	}
	
	public void setPosition(int newPosition) {
		position = newPosition;
	}

	public void setLine(int newLine) {
		line = newLine;
	}
	
	public abstract INode clone();

}
