/**
 *  This file is part of
 *  
 *  Solenopsis 2: A Framework for the Development of Ant Algorithms
 *  
 *  Copyright (C) 2006-2009 Amos Brocco <amos.brocco@unifr.ch>
 *  						Pervasive and Artificial Intelligence Research Group
 *  						Department of Informatics DIUF
 *  						University of Fribourg, Fribourg
 *  						Switzerland
 *
 *  Solenopsis 2 is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  Solenopsis 2 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with Solenopsis 2.  If not, see <http://www.gnu.org/licenses/>.
 *  
 */
package dot.preprocessor;
import java.io.BufferedReader;
import java.io.DataInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.List;

import providers.IResourceProvider;


public class Preprocessor {
	
	private static List<String> getLines(IResourceProvider rp, String filename) throws Exception {
		InputStream fstream;
		try {
			fstream = rp.getResourceInputStream(filename);
			return getLines(fstream);
		} catch (Exception e) {
			throw new Exception(String.format("Error reading file %s", filename));
		}
	}
	
	private static List<String> getLines(InputStream stream) throws Exception {
		DataInputStream in = new DataInputStream(stream);
        BufferedReader br = new BufferedReader(new InputStreamReader(in));
        String line;
        ArrayList<String> lines = new ArrayList<String>();
        try {
			while ((line = br.readLine()) != null)
				lines.add(line+"\n");
		} catch (IOException e) {
			throw new Exception("Error reading file");
		} finally {
			stream.close();
		}
		
		return lines;
	}
	
	private static List<String> processIncludes(IResourceProvider rp, List<String> code) throws Exception {
		List<String> output = new ArrayList<String>();
		boolean cleanRun = false;
		
		while (!cleanRun) {
			cleanRun = true;
			output = new ArrayList<String>();
			for (String l : code) {
				String ltrimmed = l.trim();
				int llength = ltrimmed.length();
				String lupper = ltrimmed.toUpperCase();
				if ((llength >= 8) && (lupper.substring(0, 8).equals("#INCLUDE"))) {
					cleanRun = false;
					String inc = cleanName(ltrimmed.substring(8));
					output.addAll(getLines(rp, inc));
				} else if ((llength >= 12) && (lupper.substring(0, 8).equals("#ENCAPSULATE"))) {
					cleanRun = false;
					String enc = cleanName(ltrimmed.substring(12));
					List<String> encapsulated = getLines(rp, enc);
					output.add("(let () (begin\n");
					output.addAll(encapsulated);
					output.add("))\n");
				} else {
					output.add(l);
				}
			}
			code = output;	
		}
		return output;
	}
	
	
	public static int preprocess(IResourceProvider rp, InputStream dslis, InputStream istream, List<String> code) throws Exception{
		/* Include DSL definitions */
		List<String> dslCode = processIncludes(rp, getLines(dslis));
		code.addAll(dslCode);
		
		/* Process file */
		List<String> realcode = processIncludes(rp, getLines(istream));
		code.addAll(realcode);	
		
		return dslCode.size();
	}

	private static String cleanName(String oname) throws Exception {
		String name = oname.trim();
		if (name.charAt(0) == '"') {
			name = name.substring(1);
		} else {
			throw new Exception(String.format("Missing initial quote %s",oname));
		}
		if (name.charAt(name.length()-1) == '"') {
			name = name.substring(0,name.length()-1);
		} else {
			throw new Exception(String.format("Missing final quote %s",oname));
		}
		return name;
	}
	

}
