/**
 *  This file is part of
 *  
 *  Solenopsis 2: A Framework for the Development of Ant Algorithms
 *  
 *  Copyright (C) 2006-2009 Amos Brocco <amos.brocco@unifr.ch>
 *  						Pervasive and Artificial Intelligence Research Group
 *  						Department of Informatics DIUF
 *  						University of Fribourg, Fribourg
 *  						Switzerland
 *
 *  Solenopsis 2 is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  Solenopsis 2 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with Solenopsis 2.  If not, see <http://www.gnu.org/licenses/>.
 *  
 */
package interfaces.scheduling;

import java.util.UUID;

import com.fuzzbug.solenopsis.sdk.scheduling.IAlarmable;



public interface IAlarmService {

	/**
	 * Schedules a periodic task
	 * @param client	The alarm client
	 * @param interval	Interval between callbacks
	 * @param async	Asynchronous flag
	 * @return			Identifier of the task
	 */
	public UUID schedule(IAlarmable client, Object data, long interval, boolean async);
	
	/**
	 * Schedules a periodic task
	 * @param client	The alarm client
	 * @param interval	Interval between callbacks
	 * @param firsttime Timestamp for the first execution
	 * @param async	Asynchronous flag
	 * @return			Identifier of the task
	 */
	public UUID scheduleIn(IAlarmable client, Object data, long interval, long firsttime, boolean async);
	
	/**
	 * Schedules "one-shot" task
	 * @param client		The alarm client
	 * @param timestamp	Timestamp for the callback
	 * @param async		Asynchronous flag
	 * @return				Identifier of the task
	 */
	public UUID scheduleAt(IAlarmable client, Object data, long timestamp, boolean async);
	
	/**
	 * Unschedules a task
	 * @param taskId	The task identifier
	 */
	public void unschedule(UUID taskId);
	
	/**
	 * Unschedules all tasks for the given client
	 * @param client	The client object
	 */
	public void unscheduleAll(IAlarmable client);
	
	/**
	 * Returns true if the client has some task scheduled
	 * @param client	The client object
	 * @return			True if the client has some tasks scheduled
	 */
	public boolean hasSchedule(IAlarmable client);
	
	/**
	 * Returns a list of task identifiers associated to the client
	 * @param client	The client object
	 * @return			List of task identifiers
	 */
	public UUID[] getSchedule(IAlarmable client);
	
}
