/**
 *  This file is part of
 *  
 *  Solenopsis 2: A Framework for the Development of Ant Algorithms
 *  
 *  Copyright (C) 2006-2009 Amos Brocco <amos.brocco@unifr.ch>
 *  						Pervasive and Artificial Intelligence Research Group
 *  						Department of Informatics DIUF
 *  						University of Fribourg, Fribourg
 *  						Switzerland
 *
 *  Solenopsis 2 is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  Solenopsis 2 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with Solenopsis 2.  If not, see <http://www.gnu.org/licenses/>.
 *  
 */
package interfaces.vm.valuetypes;


import java.io.Serializable;
import java.util.List;
import java.util.Map;

import com.fuzzbug.solenopsis.sdk.vm.IVMCode;
import com.fuzzbug.solenopsis.sdk.vm.IVMStack;



public interface ILambdaValue extends Serializable {
	
	/**
	 * Returns the absolute position (VMOp index) in the bytecode
	 * where the Lambda code starts.
	 * @return	Number value with the absolute start instruction index
	 */
	public Integer getLambdaStart();
	
	/**
	 * Returns a list of IStringValue representing names of variables bound to
	 * this Lambda object
	 * @return	A list of IStringValue with the name of bound variables
	 */
	public List<Object> getVariables();
	
	/**
	 * Returns the value bound to the given variable
	 * @param identifier	The variable name
	 * @return				The value bound to the variable in the Lambda object
	 */
	public Object getVariableValue(String identifier);
	
	/**
	 * Returns a dictionary with the variable name / value bindings
	 * @return		Dictionary of variable bindings
	 */
	public Map<String,Object> getBindings();
	
	/**
	 * Updates the bindings according to the given state
	 * @param env	The stack whereto retrieve current bindings
	 */
	public void updateBindings(IVMStack env);

	public IVMCode getCode();
	
}
