/**
 *  This file is part of
 *  
 *  Solenopsis 2: A Framework for the Development of Ant Algorithms
 *  
 *  Copyright (C) 2006-2009 Amos Brocco <amos.brocco@unifr.ch>
 *  						Pervasive and Artificial Intelligence Research Group
 *  						Department of Informatics DIUF
 *  						University of Fribourg, Fribourg
 *  						Switzerland
 *
 *  Solenopsis 2 is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  Solenopsis 2 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with Solenopsis 2.  If not, see <http://www.gnu.org/licenses/>.
 *  
 */
package logging;

import dot.codegenerator.ICodeGenerator;
import dot.parser.nodes.INode;


public class Logger {
	
	public static void CompilerFatal(INode node, ICodeGenerator cg, String message, Object... oo) throws Exception {
		String context = cg == null ? "<parser>" : String.format("<cg:%s>", cg.getContext());
		Logger.fatal("Compiler fatal: %s @ %s, file %s, context %s, line %d, position %d : %s", String.format(message, oo), node.toString(), node.getFilename(), context, node.getLine(),
				node.getPosition(), node.getCode());
	}
	
	public static void CompilerWarning(INode node, ICodeGenerator cg, String message, Object... oo) throws Exception {
		String context = cg == null ? "<parser>" : String.format("<cg:%s>", cg.getContext());
		Logger.fatal("Compiler warning: %s @ %s, file %s, context %s, line %d, position %d : %s", String.format(message, oo), node.toString(), node.getFilename(), context, node.getLine(),
				node.getPosition(), node.getCode());
	}
	
	public static void fatal(String message, Object... oo) throws Exception {
		Logger.fatal(String.format(message, oo));
	}
	
	public static void warning(String message, Object... oo) {
		Logger.warning(String.format(message, oo));
	}

	public static void fatal(String message) throws Exception {	
		new Exception().printStackTrace();
		throw new Exception(message);
	}
	
	public static void warning(String message) {
		StaticLogger.log(3, message);
	}

	public static void info(String message, Object... oo)  {
		StaticLogger.log(2 ,String.format(message, oo));
	}

	public static void debug(String message, Object... oo) {
		StaticLogger.log(0, String.format(message, oo));
	}
	
	


}
