package opt.blatant;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.Stack;
import java.util.concurrent.BrokenBarrierException;
import java.util.concurrent.CyclicBarrier;

public class ParallelCompute {

	private int diameter = 0;
	private int radius = 0;
	private CyclicBarrier b;
	private List<Integer> lengths;
	private float apl = 0;
	private int largestpartition = 0;
	private int smallestpartition = 100000;

	private class Computer implements Runnable {

		public void run() {
			String job = null;
			for (;;) {
				job = getJob();
				if (job == null)
					break;
				Map<String, Integer> dist = distancesFromSource(job);
				int retval = Collections.max(dist.values());
				pushResult(retval);
			}
			try {
				b.await();
			} catch (InterruptedException e) {
				e.printStackTrace();
			} catch (BrokenBarrierException e) {
				e.printStackTrace();
			}
		}
	}

	private Map<String, Integer> distancesFromSource(String source) {
		Map<String, Integer> visited = new Hashtable<String, Integer>();
		Set<String> toVisit = new HashSet<String>();
		Set<String> visiting = new HashSet<String>();
		int distance = 0;
		int nodesvisited = 0;

		toVisit.add(source);

		while (!toVisit.isEmpty()) {
			visiting.clear();
			visiting.addAll(toVisit);
			toVisit.clear();
			for (String enne : visiting) {
				String n = new String(enne);
				if (!(visited.containsKey(n))) {
					visited.put(n, new Integer(distance));
					BlatAntLogic node = NetworkSystem.getNode(n);
					if (node != null) {
						nodesvisited++;
						node.acquire();
						for (String k : node.getNeighbors()) {
							BlatAntLogic u = NetworkSystem.getNode(k);
							if (u != null)
								if (u.getNeighbors().contains(n))
									toVisit.add(new String(k));
						}
						node.release();
					}
				}
			}
			distance++;
		}

		/*if (false && (nodesvisited - NetworkSystem.getNodeCount() >= 0)) {
			System.out
					.format(
							"Network is partitioned! %d nodes, %d visited by BFS; job from %s, neighbors %d \n",
							NetworkSystem.getNodeCount(), nodesvisited, source,
							NetworkSystem.getNode(source).getNeighborsCount());
		}*/
		partition(nodesvisited);
		return visited;
	}
	
	private synchronized void partition(int nodesvisited) {
		if (nodesvisited > largestpartition)
			largestpartition = nodesvisited;
		if (nodesvisited < smallestpartition)
			smallestpartition = nodesvisited;
	}

	private Stack<String> nodes;
	private int results;
	private int threadcount;

	public ParallelCompute(int threadcount) {
		nodes = new Stack<String>();
		b = new CyclicBarrier(threadcount + 1);
		this.threadcount = threadcount;
		lengths = new ArrayList<Integer>();
	}

	private synchronized String getJob() {
		if (nodes.isEmpty())
			return null;
		return nodes.pop();
	}

	private synchronized void pushResult(int val) {
		lengths.add(val);
		diameter = Math.max(diameter, val);
		radius = Math.min(radius, val);
		results--;
	}

	public int largest() {
		return largestpartition;
	}
	
	public int smallest() {
		return smallestpartition;
	}
	
	public void computeBFS() {
		nodes.clear();
		nodes.addAll(NetworkSystem.getNodes());
		results = nodes.size();
		radius = Integer.MAX_VALUE;
		lengths.clear();
		diameter = 0;
		largestpartition = 0;
		smallestpartition = 100000;
		for (int k = 0; k < threadcount; k++) {
			new Thread(new Computer()).start();
		}
		try {
			b.await();
		} catch (InterruptedException e) {
			e.printStackTrace();
		} catch (BrokenBarrierException e) {
			e.printStackTrace();
		}
		computeAPL();
	}

	public int getDiameter() {
		return diameter;
	}

	public int getRadius() {
		return radius;
	}

	private void computeAPL() {
		apl = 0;
		for (int i : lengths) {
			apl = apl + (float) i;
		}
		apl = apl / (float) lengths.size();
	}

	public float getAveragePathLength() {
		return apl;
	}

}
