/**
 *  This file is part of
 *  
 *  Solenopsis 2: A Framework for the Development of Ant Algorithms
 *  
 *  Copyright (C) 2006-2009 Amos Brocco <amos.brocco@unifr.ch>
 *  						Pervasive and Artificial Intelligence Research Group
 *  						Department of Informatics DIUF
 *  						University of Fribourg, Fribourg
 *  						Switzerland
 *
 *  Solenopsis 2 is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  Solenopsis 2 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with Solenopsis 2.  If not, see <http://www.gnu.org/licenses/>.
 *  
 */
package opt.blatant;

import java.util.Hashtable;
import java.util.LinkedList;
import java.util.List;

public class Pheromone {

	private Hashtable<String, Double> trails;
	private Double defvalue;
	private Double decay = 0.9;
	private Double increment = 0.1;
	private Double minvalue = 0.005;
	private List<String> d = new LinkedList<String>();
	private String name;


	public Pheromone(String name, Double decay, Double increment, Double defvalue, Double minvalue) {
		trails = new Hashtable<String, Double>(200);
		this.decay = decay;
		this.increment = increment;
		this.defvalue = defvalue;
		this.minvalue = minvalue;
		this.name = name;
	}

	public synchronized void clear(String trail) {
		trails.remove(trail);
	}
	
	public String getName() {
		return name;
	}

	public synchronized List<String> decay() {
		double value;
		d.clear();
		for (String trail : trails.keySet()) {
			value = trails.get(trail) * decay;
			//System.out.println(name+" :: "+trail+" ---> "+ value+" < "+minvalue);
			if (value < minvalue) {
				d.add(trail);
			} else {
				trails.put(trail, value);
			}
		}
		for (String t : d) {
			trails.remove(t);
		}
		return d;
	}

	public synchronized void deposit(String trail) {
		Double value = trails.get(trail);
		if (value == null) {
			init(trail);
			value = defvalue;
		}
		trails.put(trail, increment(value));
	}

	public synchronized void fill(String trail) {
		trails.put(trail, defvalue);
	}

	public synchronized void remove(String trail) {
		if (trail == null)
			return;
		Double value = trails.get(trail);
		if (value == null)
			value = defvalue;
		trails.put(trail, decrement(value));
	}

	private double increment(double value) {
		return value + increment;
	}

	private double decrement(double value) {
		return value - increment;
	}

	private void init(String trail) {
		trails.put(trail, defvalue);
	}

	public synchronized double senseTrail(String trail) {
		Double value = trails.get(trail);
		if (value == null)
			return 0;
		else
			return value;
	}

	public void clearAll() {
		trails.clear();
		d.clear();
	}

}
