/**
 *  This file is part of
 *  
 *  Solenopsis 2: A Framework for the Development of Ant Algorithms
 *  
 *  Copyright (C) 2006-2009 Amos Brocco <amos.brocco@unifr.ch>
 *  						Pervasive and Artificial Intelligence Research Group
 *  						Department of Informatics DIUF
 *  						University of Fribourg, Fribourg
 *  						Switzerland
 *
 *  Solenopsis 2 is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  Solenopsis 2 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with Solenopsis 2.  If not, see <http://www.gnu.org/licenses/>.
 *  
 */
package opt.blatant;

import java.util.Iterator;

import scheduling.NodeSchedulingService;

public class Table implements Iterable<Entry> {

	private Entry[] table;

	public Table(int size) {
		table = new Entry[size];
	}

	public void clear() {
		for (int i = 0; i < table.length; i++)
			table[i] = null;
	}

	private int create(int i, String e) {
		if ((table[i] == null) || (table[i].isFree()))
			table[i] = new Entry(e);
		return i;
	}

	private int freeOldestIndex() {
		long timestamp = NodeSchedulingService.instance().getTime();
		long age;
		int index = -1;
		for (int i = 0; i < table.length; i++) {
			if (table[i] == null) {
				index = i;
				break;
			}
			// As we are called by getFreeIndex no entry is null or free
			if (table[i].isSticky())
				continue;
			age = table[i].getEntryAge();
			if (age < timestamp) {
				timestamp = age;
				index = i;
			}
		}
		table[index].free();
		return index;
	}

	public Entry get(int i) {
		return table[i];
	}

	public Entry get(String id) {
		int z = getIndex(id);
		return z < 0 ? null : table[z];
	}

	public Entry getOrNew(String id) {
		int i = getIndexOrNew(id);
		return get(i);
	}

	public int getFreeIndex() {
		for (int i = 0; i < table.length; i++) {
			if ((table[i] == null) || (table[i].isFree()))
				return i;
		}
		return freeOldestIndex();
	}

	public int getIndex(String id) {
		for (int i = 0; i < table.length; i++) {
			if ((table[i] != null) && (!table[i].isFree())
					&& (table[i].getId().equals(id)))
				return i;
		}
		return -1;
	}

	public int getIndexOrNew(String id) {
		int i = getIndex(id);
		return i == -1 ? create(getFreeIndex(), id) : i;
	}

	public Iterator<Entry> iterator() {
		return new Iterator<Entry>() {

			private int Index = 0;

			public boolean hasNext() {
				return Index < size();
			}

			public Entry next() {
				return table[Index++];
			}

			public void remove() {
				throw new UnsupportedOperationException();
			}
		};
	}

	public int size() {
		return table.length;
	}

	public void stick(int i) {
		if (i >= 0)
			table[i].stick();
	}

	public void unstick(int i) {
		if (i >= 0) {
			table[i].unstick();
		}
	}

	public void updated(int i, Entry e) {
		if (table[i] == null)
			table[i] = new Entry(e.getId());
		table[i].updateFrom(e);
	}

	public boolean contains(String id) {
		return (get(id) != null) && (!get(id).isFree());
	}

}
