/**
 *  This file is part of
 *  
 *  Solenopsis 2: A Framework for the Development of Ant Algorithms
 *  
 *  Copyright (C) 2006-2009 Amos Brocco <amos.brocco@unifr.ch>
 *  						Pervasive and Artificial Intelligence Research Group
 *  						Department of Informatics DIUF
 *  						University of Fribourg, Fribourg
 *  						Switzerland
 *
 *  Solenopsis 2 is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  Solenopsis 2 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with Solenopsis 2.  If not, see <http://www.gnu.org/licenses/>.
 *  
 */
package providers;

import interfaces.exe.ICodeProvider;

import java.io.InputStream;
import java.util.Hashtable;
import java.util.List;
import java.util.Observable;

import com.fuzzbug.solenopsis.sdk.lib.IInstruction;

import dot.compiler.Compiler;

public class CodeProvider extends Observable implements ICodeProvider {

	private Hashtable<String, List<IInstruction>> cache = new Hashtable<String, List<IInstruction>>();

	public List<IInstruction> getCode(String path) throws Exception {
			InputStream is = null;
			if (cache.containsKey(path)) {
				return cache.get(path);
			} else {
				List<IInstruction> ilist = null;
				try {
					is = DefaultResourceProvider.instance().getResourceInputStream(path);
					ilist = Compiler.compile(DefaultResourceProvider.instance(), path, is);
					cache.put(path, ilist);
				} catch (Exception e) {
					throw e;
				} finally {
					if (is != null)
						is.close();
				}
				return ilist;
			}
	}

	@Override
	public void invalidate(String path) {
		cache.remove(path);
	}

}
