/**
 *  This file is part of
 *  
 *  Solenopsis 2: A Framework for the Development of Ant Algorithms
 *  
 *  Copyright (C) 2006-2009 Amos Brocco <amos.brocco@unifr.ch>
 *  						Pervasive and Artificial Intelligence Research Group
 *  						Department of Informatics DIUF
 *  						University of Fribourg, Fribourg
 *  						Switzerland
 *
 *  Solenopsis 2 is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  Solenopsis 2 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with Solenopsis 2.  If not, see <http://www.gnu.org/licenses/>.
 *  
 */
package providers;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;

public class FileResourceProvider implements IResourceProvider {

	private File resBasePath;

	public FileResourceProvider(String path) {
		resBasePath = new File(path);
	}

	private File getFile(String resource) {
		if (resource == null)
			return null;
		File rfile = new File(resource);
		if (rfile.isAbsolute()) {
			return rfile;
		} else {
			return new File(resBasePath, resource);
		}
	}

	@Override
	public InputStream getResourceInputStream(String resource) throws Exception {
		if (resource == null)
			return null;
		File in = getFile(resource);
		InputStream is = null;
		is = new BufferedInputStream(new FileInputStream(in.getAbsolutePath()));
		return is;
	}
	
	public String toString() {
		return String.format("FileResourceProvider:%s", resBasePath);
	}

}
