/**
 *  This file is part of
 *  
 *  Solenopsis 2: A Framework for the Development of Ant Algorithms
 *  
 *  Copyright (C) 2006-2009 Amos Brocco <amos.brocco@unifr.ch>
 *  						Pervasive and Artificial Intelligence Research Group
 *  						Department of Informatics DIUF
 *  						University of Fribourg, Fribourg
 *  						Switzerland
 *
 *  Solenopsis 2 is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  Solenopsis 2 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with Solenopsis 2.  If not, see <http://www.gnu.org/licenses/>.
 *  
 */
package providers;

import java.io.InputStream;

public class GenericResourceProvider implements IResourceProvider {

	private IResourceProvider urp;
	private IResourceProvider frp;
	private String path;
	
	public GenericResourceProvider(String path) {
		this.path = path;
		urp = new URLResourceProvider();
		frp = new FileResourceProvider(path);
	}
	
	@Override
	public InputStream getResourceInputStream(String resource) throws Exception {
		if (resource.startsWith("http")) {
			return urp.getResourceInputStream(resource);
		} else
			return frp.getResourceInputStream(resource);
	}
	
	public String toString() {
		return String.format("Generic resource provider from %s", path);
	}
	
	

}
