/**
 *  This file is part of
 *  
 *  Solenopsis 2: A Framework for the Development of Ant Algorithms
 *  
 *  Copyright (C) 2006-2009 Amos Brocco <amos.brocco@unifr.ch>
 *  						Pervasive and Artificial Intelligence Research Group
 *  						Department of Informatics DIUF
 *  						University of Fribourg, Fribourg
 *  						Switzerland
 *
 *  Solenopsis 2 is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  Solenopsis 2 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with Solenopsis 2.  If not, see <http://www.gnu.org/licenses/>.
 *  
 */
package providers;

import interfaces.exe.ICodeProvider;

import java.io.InputStream;

import com.fuzzbug.solenopsis.sdk.exe.INodeDaemon;
import com.fuzzbug.solenopsis.sdk.vm.IVM;
import com.fuzzbug.solenopsis.sdk.vm.IVMState;


import core.signals.exceptions.InvalidCallbackException;
import core.signals.exceptions.InvalidSignalException;
import core.vm.VM;
import core.vm.lib.VMCode;
import core.vm.lib.VMStack;
import core.vm.lib.VMState;

public class VMFactory {
	
	private static void connectVMSignals (VM vm, INodeDaemon daemon) {
		try {
			/* Connects signals that can be emitted by the VM */
			vm.connect("extern-op-call", daemon, "vmExternOpCall");
			vm.connect("request-stop", daemon, "vmStopRequest");
		} catch (InvalidCallbackException e) {
		} catch (InvalidSignalException e) {
		}
	}

	public static IVM getVM(ICodeProvider c, InputStream stream, INodeDaemon daemon) throws Exception {
		IVMState state = new VMState(c, stream);
		return getVM(state, daemon);
	}

	public static IVM getVM(IVMState state, INodeDaemon daemon) {
		VM vm = new VM(state, daemon.getIdentifier());
		connectVMSignals (vm, daemon);
		return vm;
	}

	public static IVM getVM(ICodeProvider c, String source, INodeDaemon daemon) throws Exception {
		IVMState state = new VMState(new VMStack(), new VMCode(c, source), 0);
		return getVM(state, daemon);
	}

}
