/**
 *  This file is part of
 *  
 *  Solenopsis 2: A Framework for the Development of Ant Algorithms
 *  
 *  Copyright (C) 2006-2009 Amos Brocco <amos.brocco@unifr.ch>
 *  						Pervasive and Artificial Intelligence Research Group
 *  						Department of Informatics DIUF
 *  						University of Fribourg, Fribourg
 *  						Switzerland
 *
 *  Solenopsis 2 is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  Solenopsis 2 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with Solenopsis 2.  If not, see <http://www.gnu.org/licenses/>.
 *  
 */
package scheduling;


import java.util.UUID;

import com.fuzzbug.solenopsis.sdk.scheduling.IAlarmable;
import com.fuzzbug.solenopsis.sdk.scheduling.INodeAlarmService;



public class NodeSchedulingService implements Runnable, INodeAlarmService {
	
	private AlarmService as = new AlarmService(false);
	
	private static NodeSchedulingService instance;
	
	private long RESOLUTION_TIME_MS = 1;
	
	public synchronized static INodeAlarmService instance() {
		if (instance == null) {
			instance = new NodeSchedulingService();
			new Thread(instance).start();
		}
		return instance;
	}

	@Override
	public void run() {
		for (;;) {
			try {
				Thread.sleep(RESOLUTION_TIME_MS);
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
			as.processAlarms();
			Thread.yield();
		}
	}
	
	/* (non-Javadoc)
	 * @see scheduling.INodeSchedulingService#scheduleAt(interfaces.scheduling.IAlarmable, java.lang.Object, long)
	 */
	public UUID scheduleAt(IAlarmable t, Object data, long time) {
		return as.scheduleAt(t, data, time, false);
	}
	
	/* (non-Javadoc)
	 * @see scheduling.INodeSchedulingService#schedule(interfaces.scheduling.IAlarmable, java.lang.Object, long)
	 */
	public UUID schedule(IAlarmable t, Object data, long interval) {
		return as.schedule(t, data, interval, false);
	}
	
	/* (non-Javadoc)
	 * @see scheduling.INodeSchedulingService#scheduleAfter(interfaces.scheduling.IAlarmable, java.lang.Object, long)
	 */
	public UUID scheduleAfter(IAlarmable t, Object data, long interval) {
		return as.scheduleAfter(t, data, interval, false);
	}
	
	/* (non-Javadoc)
	 * @see scheduling.INodeSchedulingService#unschedule(java.util.UUID)
	 */
	public void unschedule(UUID task) {
		as.unschedule(task);
	}
	
	/* (non-Javadoc)
	 * @see scheduling.INodeSchedulingService#unschedule(interfaces.scheduling.IAlarmable)
	 */
	public void unschedule(IAlarmable alarmable) {
		as.unscheduleAll(alarmable);
	}

	@Override
	public long getTime() {
		return as.getTime();
	}

}
