/**
 *  This file is part of
 *  
 *  Solenopsis 2: A Framework for the Development of Ant Algorithms
 *  
 *  Copyright (C) 2006-2009 Amos Brocco <amos.brocco@unifr.ch>
 *  						Pervasive and Artificial Intelligence Research Group
 *  						Department of Informatics DIUF
 *  						University of Fribourg, Fribourg
 *  						Switzerland
 *
 *  Solenopsis 2 is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  Solenopsis 2 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with Solenopsis 2.  If not, see <http://www.gnu.org/licenses/>.
 *  
 */
package services;

import java.util.List;
import java.util.Map;

import com.fuzzbug.solenopsis.sdk.exe.INodeDaemon;
import com.fuzzbug.solenopsis.sdk.plugins.BasicPlugin;
import com.fuzzbug.solenopsis.sdk.vm.IVM;
import com.fuzzbug.solenopsis.sdk.vm.IVMState;

public class Ants extends BasicPlugin {
	
	public Ants(INodeDaemon d) {
		super("ants", "Amos Brocco <amos.brocco@unifr.ch>", "Plugin for creating ants", "1.0", true);
		registerMethods();
	}
	
	@SuppressWarnings("unchecked")
	public Object apply_new(INodeDaemon node, IVM vm, List<Object> args) throws Exception {
		if ((args.size() > 2) || args.isEmpty()) return false;
		IVMState state = node.getState((String) args.get(0));
		if (args.size() == 2) {
			Map<String,Object> vars = (Map<String, Object>) args.get(1);
			for (String vname : vars.keySet()) {
				state.getStack().storeGValue(vname, vars.get(vname));
			}
		}
		node.sendState(node.getIdentifier(), state);
		return false;
	}
	
	@SuppressWarnings("unchecked")
	public Object apply_send(INodeDaemon node, IVM vm, List<Object> args) throws Exception {
		if ((args.size() > 3) || args.isEmpty()) return false;
		String recipient = (String) args.get(0);
		IVMState state = node.getState((String) args.get(1));
		if (args.size() == 3) {
			Map<String,Object> vars = (Map<String, Object>) args.get(2);
			for (String vname : vars.keySet()) {
				state.getStack().storeGValue(vname, vars.get(vname));
			}
		}
		node.sendState(recipient, state);
		return false;
	}

}
