/**
 *  This file is part of
 *  
 *  Solenopsis 2: A Framework for the Development of Ant Algorithms
 *  
 *  Copyright (C) 2006-2009 Amos Brocco <amos.brocco@unifr.ch>
 *  						Pervasive and Artificial Intelligence Research Group
 *  						Department of Informatics DIUF
 *  						University of Fribourg, Fribourg
 *  						Switzerland
 *
 *  Solenopsis 2 is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  Solenopsis 2 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with Solenopsis 2.  If not, see <http://www.gnu.org/licenses/>.
 *  
 */
package services;

import interfaces.exe.ICodeProvider;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.util.List;
import java.util.Map;
import java.util.Observable;
import java.util.Observer;

import providers.VMFactory;
import bebop.client.Client;
import bebop.lib.IMessage;
import bebop.lib.Message;
import bebop.mailbox.MailBox;
import bebop.mailbox.MailService;

import com.fuzzbug.solenopsis.sdk.exe.INodeDaemon;
import com.fuzzbug.solenopsis.sdk.plugins.BasicPlugin;
import com.fuzzbug.solenopsis.sdk.vm.IVM;

import exe.NodeDaemon;

public class BebopProxy extends BasicPlugin implements Observer {
	
	private String[] tagNames = { "getLocalAddress", "sendMessage" };
	
	private MailBox mailbox;

	private MailService ms;

	private NodeDaemon d;

	private ICodeProvider c;
	
	public BebopProxy(ICodeProvider c, String host, int port, NodeDaemon d) throws IOException {
		this(c, MailService.start(host, port), d);
	}
	
	public BebopProxy(ICodeProvider c, MailService ms, NodeDaemon d) {
		super("bebop", "Amos Brocco <amos.brocco@unifr.ch", "Bebop Mailbox Proxy", "1.0", false);
		this.mailbox = ms.createMailBox(this, d.getIdentifier().toString());
		this.ms = ms;
		this.d = d;
		this.c = c;
		d.setIdentifier(ms.getAddress(mailbox));
		registerMethods(tagNames);
	}
	
	/* Method definition */

	public Object apply_getLocalAddress(INodeDaemon node, IVM vm,
			List<Object> args) throws Exception {
		return ms.getAddress(mailbox);
	}
	
	@SuppressWarnings("unchecked")
	public Object apply_sendMessage(INodeDaemon node, IVM vm,
			List<Object> args) throws Exception {
		try {
			String recipient = (String) args.get(0); // Recipient bebop address
			Map<String,Object> contents = (Map<String, Object>) args.get(1);
			Message msg = new Message(recipient, contents);
			Client.sendMessage(msg);
		} catch (Exception e) {
			e.printStackTrace();
			return false;
		}
		return 1;
	}
	
	/* Observer stuff */

	@Override
	public void update(Observable source, Object data) {
		try {
			if (source == mailbox) {
				for (;;) {
					IMessage msg = mailbox.popMessage();
					if (msg != null) {
						String contentType = (String) msg.getContents().get("type");
						if ("ant".equals(contentType)) {
							String stateSer = (String) msg.getContents().get("state");
							Integer pc = (Integer) msg.getContents().get("pc");
							ByteArrayInputStream bis = new ByteArrayInputStream(stateSer.getBytes());
							IVM vm = VMFactory.getVM(c, bis, d);
							vm.setPC(pc);
							d.schedule(vm);
						}
					} else 	break;
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	public void doFinalize() throws Exception {
		ms.destroyMailBox(d.getIdentifier().toString());
	}

}
