package services;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import util.Dolly;

import com.fuzzbug.solenopsis.sdk.exe.INodeDaemon;
import com.fuzzbug.solenopsis.sdk.plugins.BasicPlugin;
import com.fuzzbug.solenopsis.sdk.vm.IVM;

public class SharedSpace extends BasicPlugin {
	
	private Map<String,Object> sharedObjects = new HashMap<String,Object>();

	public SharedSpace(INodeDaemon d) {
		super("shared", "Amos Brocco", "Implements a linda-like tuple space", "1.0", false);
		registerMethods();
	}
	
	public Object apply_retrieve(INodeDaemon node, IVM vm,
			List<Object> args) throws Exception {
		String keyPrefix = (String) args.get(0);
		List<Object> result = new ArrayList<Object>();
		synchronized(sharedObjects) {
			List<String> keys = new ArrayList<String>();
			keys.addAll(sharedObjects.keySet());
			for (String k : keys) {
				if (k.startsWith(keyPrefix)) {
					result.add(Dolly.deepClone(sharedObjects.get(k)));
				}
			}
			return result;
		}
	}
	
	public Object apply_retrieveFirst(INodeDaemon node, IVM vm,
			List<Object> args) throws Exception {
		String keyPrefix = (String) args.get(0);
		Object result = Boolean.FALSE;
		synchronized(sharedObjects) {
			List<String> keys = new ArrayList<String>();
			keys.addAll(sharedObjects.keySet());
			for (String k : keys) {
				if (k.startsWith(keyPrefix)) {
					return Dolly.deepClone(sharedObjects.get(k));
				}
			}
			return result;
		}
	}
	
	public Object apply_pull(INodeDaemon node, IVM vm,
			List<Object> args) throws Exception {
		String keyPrefix = (String) args.get(0);
		List<Object> result = new ArrayList<Object>();
		synchronized(sharedObjects) {
			List<String> keys = new ArrayList<String>();
			keys.addAll(sharedObjects.keySet());
			for (String k : keys) {
				if (k.startsWith(keyPrefix)) {
					result.add(sharedObjects.remove(k));
				}
			}
			return result;
		}
	}
	
	public Object apply_publish(INodeDaemon node, IVM vm,
			List<Object> args) throws Exception {
		String key = (String) args.get(0);
		Object value = args.get(1);
		synchronized(sharedObjects) {
			sharedObjects.put(key, Dolly.deepClone(value));
		}
		return key;
	}
	
	

}
