/**
 *  This file is part of
 *  
 *  Solenopsis 2: A Framework for the Development of Ant Algorithms
 *  
 *  Copyright (C) 2006-2009 Amos Brocco <amos.brocco@unifr.ch>
 *  						Pervasive and Artificial Intelligence Research Group
 *  						Department of Informatics DIUF
 *  						University of Fribourg, Fribourg
 *  						Switzerland
 *
 *  Solenopsis 2 is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  Solenopsis 2 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with Solenopsis 2.  If not, see <http://www.gnu.org/licenses/>.
 *  
 */
package services;

import interfaces.vm.util.ISerializer;

import java.io.ByteArrayOutputStream;
import java.util.List;

import serialization.Serializer;

import com.fuzzbug.solenopsis.sdk.exe.INodeDaemon;
import com.fuzzbug.solenopsis.sdk.plugins.BasicPlugin;
import com.fuzzbug.solenopsis.sdk.vm.IVM;

public class StateSerialization extends BasicPlugin {
	
	private String[] tagNames = { "getState" };

	private String[] methodNames = { "apply_getstate" };
		
	public StateSerialization() {
		super("stateserialization", "Amos Brocco <amos.brocco@unifr.ch", "VM State Serialization", "1.0", true);
		registerMethods(tagNames, methodNames);
	}

	public Object apply_getstate(INodeDaemon node, IVM vm,
			List<Object> args) throws Exception {
		/* If we migrate from a lambda we need to update the variables */
		vm.getStack().updateAllLambdaBindings();
		
		ByteArrayOutputStream out = new ByteArrayOutputStream();
		ISerializer ser = new Serializer(out);
		try {
			ser.print(vm.getState().toBaseValue());
		} catch (Exception e) {
			e.printStackTrace();
		}
		out.flush();
		String state = out.toString();
		out.close();
		
		return state;
	}
	
	
	
	

}
