/**
 *  This file is part of
 *  
 *  Solenopsis 2: A Framework for the Development of Ant Algorithms
 *  
 *  Copyright (C) 2006-2009 Amos Brocco <amos.brocco@unifr.ch>
 *  						Pervasive and Artificial Intelligence Research Group
 *  						Department of Informatics DIUF
 *  						University of Fribourg, Fribourg
 *  						Switzerland
 *
 *  Solenopsis 2 is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  Solenopsis 2 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with Solenopsis 2.  If not, see <http://www.gnu.org/licenses/>.
 *  
 */
package softthreading;


import java.util.UUID;

import com.fuzzbug.solenopsis.sdk.scheduling.IAlarmable;
import com.fuzzbug.solenopsis.sdk.softthreading.ISoftThread;

import scheduling.AlarmService;

public class PreemptionMonitor implements Runnable, IAlarmable {
	
	private AlarmService as = new AlarmService(true);
	
	private static PreemptionMonitor instance;
	
	private long RESOLUTION_TIME_MS = 1;
	
	public synchronized static PreemptionMonitor instance() {
		if (instance == null) {
			instance = new PreemptionMonitor();
			new Thread(instance).start();
		}
		return instance;
	}

	@Override
	public void run() {
		for (;;) {
			try {
				Thread.sleep(RESOLUTION_TIME_MS);
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
			as.processAlarms();
			Thread.yield();
		}
	}
	
	public void preemptAfter(ISoftThread t, long ms) {
		as.scheduleAfter(this, t, ms, false);
	}

	@Override
	public void callback(UUID taskId, Object data, long timestamp) {
		((ISoftThread) data).suspend();
	}

}
