/**
 *  This file is part of
 *  
 *  Solenopsis 2: A Framework for the Development of Ant Algorithms
 *  
 *  Copyright (C) 2006-2009 Amos Brocco <amos.brocco@unifr.ch>
 *  						Pervasive and Artificial Intelligence Research Group
 *  						Department of Informatics DIUF
 *  						University of Fribourg, Fribourg
 *  						Switzerland
 *
 *  Solenopsis 2 is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  Solenopsis 2 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with Solenopsis 2.  If not, see <http://www.gnu.org/licenses/>.
 *  
 */
package softthreading;

import interfaces.softthreading.IScheduler;

import java.util.ArrayList;
import java.util.List;

import com.fuzzbug.solenopsis.sdk.softthreading.ISoftThread;


public class Scheduler implements IScheduler {
	
	private Object mutex;
	private List<ISoftThread> threads;
	private int currentThreadIndex = 0;
	protected long SCHEDULER_QUANTUM = 5;
	private PreemptionMonitor pm = PreemptionMonitor.instance();
	
	public Scheduler(long quantum) {
		mutex = new Object();
		threads = new ArrayList<ISoftThread>();
		SCHEDULER_QUANTUM = quantum;
	}
	
	public Scheduler() {
		this(5);
	}
	
	public void schedule(ISoftThread s) {
		synchronized(mutex) {
			threads.add(s);
		}
	}
	
	public int count() {
		synchronized(mutex) {
			return threads.size();
		}
	}
	
	public void unschedule(ISoftThread s) {
		synchronized(mutex) {
			threads.remove(s);
		}
	}
	
	protected ISoftThread getNextThread() {
		synchronized(mutex) {
			if (threads.isEmpty()) return null;
			currentThreadIndex = currentThreadIndex >= threads.size() - 1 ? 0 : currentThreadIndex + 1;
			return threads.get(currentThreadIndex);
		}
	}
	
	public boolean execute() {
		final ISoftThread s = getNextThread();
		if (s == null) { Thread.yield(); return false; }
		if (s.getInterruptibleStatus() == ISoftThread.RUNNABLE) {
			pm.preemptAfter(s, SCHEDULER_QUANTUM);
			s.run();
		}
		if (s.getInterruptibleStatus() == ISoftThread.STOPPED) {
			unschedule(s);
		}
		return true;
	}

}
