/**
 *  This file is part of
 *  
 *  Solenopsis 2: A Framework for the Development of Ant Algorithms
 *  
 *  Copyright (C) 2006-2009 Amos Brocco <amos.brocco@unifr.ch>
 *  						Pervasive and Artificial Intelligence Research Group
 *  						Department of Informatics DIUF
 *  						University of Fribourg, Fribourg
 *  						Switzerland
 *
 *  Solenopsis 2 is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  Solenopsis 2 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with Solenopsis 2.  If not, see <http://www.gnu.org/licenses/>.
 *  
 */
package softthreading;

import com.fuzzbug.solenopsis.sdk.softthreading.IInterruptible;
import com.fuzzbug.solenopsis.sdk.softthreading.ISoftThread;

public abstract class SimpleSoftThread implements ISoftThread {
	
	protected 	int 	state = IInterruptible.RUNNABLE;
	private		Object		stateMutex = new Object();
	
	protected boolean resume() {
		synchronized(stateMutex) {
			if (state == IInterruptible.RUNNABLE) {
				state = IInterruptible.RUNNING;
				return true;
			}
		}
		return false;
	}
	
	protected void runnable() {
		synchronized(stateMutex) {
			if (state == IInterruptible.SUSPENDED) {
				state = IInterruptible.RUNNABLE;
			}
		}
	}
	
	public void sleep() {
		synchronized(stateMutex) {
			state = IInterruptible.SLEEPING;
		}
	}
	
	public void wakeup() {
		synchronized(stateMutex) {
			if (state == IInterruptible.SLEEPING) {
				state = IInterruptible.RUNNABLE;
			}
		}
	}

	public void suspend() {
		if (state != IInterruptible.RUNNING) return;
		synchronized(stateMutex) {
			if (state == IInterruptible.RUNNING) {
				state = IInterruptible.SUSPENDED;
			}
		}
	}
	
	public void stop() {
		synchronized(stateMutex) {
			state = IInterruptible.STOPPED;
		}
	}
	
	public int getInterruptibleStatus() {
		return state;
	}
	
	@Override
	public void run() {
		if (!resume()) return; // If we are runnable we can continue, otherwise not
		while (state == IInterruptible.RUNNING) {
			try {
				if (!runextended()) { Thread.sleep(1); suspend(); } 
			} catch (Throwable e) {
				e.printStackTrace();
				break;
			}
		}
		runnable(); // Here we become again runnable
		Thread.yield();
	}
	
	public abstract boolean runextended();
	

}
