/**
 *  This file is part of
 *  
 *  Solenopsis 2: A Framework for the Development of Ant Algorithms
 *  
 *  Copyright (C) 2006-2009 Amos Brocco <amos.brocco@unifr.ch>
 *  						Pervasive and Artificial Intelligence Research Group
 *  						Department of Informatics DIUF
 *  						University of Fribourg, Fribourg
 *  						Switzerland
 *
 *  Solenopsis 2 is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  Solenopsis 2 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with Solenopsis 2.  If not, see <http://www.gnu.org/licenses/>.
 *  
 */
package com.fuzzbug.solenopsis.sdk.plugins;


import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import com.fuzzbug.solenopsis.sdk.exe.INodeDaemon;
import com.fuzzbug.solenopsis.sdk.vm.IVM;


public abstract class BasicPlugin implements IPlugin {

	protected String[] __tagNames = {};
	
	protected HashMap<String, Method> methods;

	@SuppressWarnings("unchecked")
	public Class[] signature = { INodeDaemon.class, IVM.class, List.class };

	protected String namespace = "";

	protected String author = "";

	protected String description = "";

	protected String version = "";

	protected boolean shareable;

	public BasicPlugin(String namespace, String author, String description, String version, boolean shareable) {
		methods = new HashMap<String, Method>();
		this.namespace = namespace;
		this.author = author;
		this.description = description;
		this.version = version;
		this.shareable = shareable;
	}

	protected void registerMethods(String[] tags, String[] applyMethods) {
		for (int i = 0; i < tags.length; i++) {
			Method m;
			String methodName = applyMethods[i];
			String tag = tags[i];
			try {
				m = this.getClass().getMethod(methodName, signature);
				methods.put(tag, m);
			} catch (SecurityException e) {
				e.printStackTrace();
			} catch (NoSuchMethodException e) {
				e.printStackTrace();
			}
		}
		__tagNames = tags;
	}
	
	protected void registerMethods(String[] tags) {
		String[] applyMethods = new String[tags.length];
		for (int i = 0; i < tags.length; i++) {
			applyMethods[i] = "apply_"+tags[i];
		}
		registerMethods(tags, applyMethods);
	}
	
	protected void registerMethods() {
		List<String> lmethods = new ArrayList<String>();
		List<String> ltags = new ArrayList<String>();
		for (Method m : this.getClass().getMethods()) {
			if (m.getName().startsWith("apply_")) {
				lmethods.add(m.getName());
				ltags.add(m.getName().substring(6));
			}
		}
		String[] applyMethods = new String[lmethods.size()];
		String[] tags = new String[lmethods.size()];
		lmethods.toArray(applyMethods);
		ltags.toArray(tags);
		registerMethods(tags, applyMethods);
	}

	public Method getMethod(String tag) {
		return methods.get(tag);
	}

	public boolean isShareable() {
		return shareable;
	}

	public String getNamespaceName() {
		return namespace;
	}

	public String[] getTagsNames() {
		return __tagNames;
	}

	public String getAuthor() {
		return author;
	}

	public String getDescription() {
		return description;
	}

	public String getVersionString() {
		return version;
	}

	public Object apply(Method m, INodeDaemon node, IVM vm, List<Object> args)
			throws Exception {
		Object result = null;
		Object[] fullArgs = new Object[2];
		fullArgs[0] = node;
		fullArgs[1] = args;
		result = m.invoke(this, fullArgs);
		return result;
	}

	public void doInitialize(INodeDaemon node) throws Exception {
	}

	public void doFinalize() throws Exception {

	}


}
