/**
 *  This file is part of
 *  
 *  Solenopsis 2: A Framework for the Development of Ant Algorithms
 *  
 *  Copyright (C) 2006-2009 Amos Brocco <amos.brocco@unifr.ch>
 *  						Pervasive and Artificial Intelligence Research Group
 *  						Department of Informatics DIUF
 *  						University of Fribourg, Fribourg
 *  						Switzerland
 *
 *  Solenopsis 2 is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  Solenopsis 2 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with Solenopsis 2.  If not, see <http://www.gnu.org/licenses/>.
 *  
 */
package com.fuzzbug.solenopsis.sdk.vm;

import interfaces.vm.valuetypes.ILambdaValue;

import java.util.Set;

import com.fuzzbug.solenopsis.sdk.lib.IBaseSerializable;




public interface IVMStackFrame extends IBaseSerializable {

	/**
	 * Returns the current program counter
	 * @return	The program counter
	 */
	public int getReturnPC();
	
	/**
	 * Returns the lambda object associated to the stack frame.
	 * Returns null if the frame is not associated to a lambda.
	 * @return	A lambda object or null
	 */
	public ILambdaValue getLambda();

	/**
	 * Pops a value from the local stack
	 * @return	The value popped
	 */
	public Object popValue();

	/**
	 * Pushes a value on the local stack
	 * @param argument	The value to be pushed
	 */
	public void pushValue(Object argument);

	/**
	 * Sets the value of a local variable; creates the variable if it does
	 * not exist.
	 * @param name		Name of the variable
	 * @param value	Value of the variable
	 */
	public void setValue(String name, Object value);

	/**
	 * Returns the value of a local variable
	 * @param name	Name of the variable
	 * @return	The value of the variable, or null if the variable is not defined
	 */
	public Object getValue(String name);

	/**
	 * Returns the value on top of the local stack
	 * @return	The value on top of the stack
	 */
	public Object getTOS();


	public boolean isEmpty();

	public boolean isLocal();
	
	public int getSize();

	public Set<String> getVariables();

}
